--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-project.adb,v $
--| Description     : project class.
--| Author          : Michael Erdmann
--| Created On      : 4.1.2000
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.1 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This module contains the project class. The object maintains the 
--| complete view of a project.
--|
--| Project class is an implementation of the dbelement class.
--|
--| Restrictions
--| ============
--| None
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: dbs-project.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;

--- Ascl
with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;			   use ASCL.DB;
use  ASCL;

--- Database objects
with DBS.Element;	            use DBS.Element;
with DBS.SQL;			    use DBS.SQL;

--- Common support procedures

package body DBS.ACDE is

   Version : constant String := "$Id: dbs-project.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "dbs-acde.adb";

   ---====================================================================---
   ---===                 C L A S S    D A T A			       ===---
   ---====================================================================---

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record

         Debug   : Debugging_Support.Handle := null;
      end record;
   			
   ---======================================================================---
   ---===            L O C A L  S U P P O R T   T Y P E S                ===---
   ---======================================================================---

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object; info : String ) return  Object_Data_Access is
   begin
      if this.Data = null then
         raise ASCL.Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;
   			
   ---======================================================================---
   ---===                 C L A S S   I N T E R F A C E                  ===---
   ---======================================================================---

   			
   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the Initialization
   --| procedure for the extension and any other object data
   --| component.
   --|
   --| If the data for the class data is not yet set 
   --|
   procedure Initialize( this  : in out Object;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise ASCL.Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      DBS.Element.Initialize( this, debug );
      Data.Debug := debug; 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

      DBS.Element.Finalize( this );

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| This function returns the first name part of the real name
   --|
   function Name( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Name");
   begin
      return "";
   end Name;


   --|
   --| Set some attribute
   --|
   procedure Name( this : in out Object; name : in Unbounded_String ) is
      Data  : Object_Data_Access := Instance( this, ".Name");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Name=" & To_String(name) );

      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Name" );
         raise;
   end Name;
   ---=====================================================================---
   ---===                C L A S S   M E T H O D S                      ===---
   ---=====================================================================---

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Load the user data for a given login name into the 
   --| object instance.
   --|
   function Load_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );   
   begin
      Enter( debug, Module & ".Load_Query");


      Leave( debug, "");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Load_Query;

   --|
   --| Store the object information in the data base. If update fails
   --| perform an insert. If this fails as well raise an exception.
   --|
   function Store_Query ( this : in Object) return Boolean is
      Data        : Object_Data_Access := Instance( this, ".Store_Query");
      Debug       : Debugging_Support.Handle renames data.Debug;
      qID         : DB.Query_ID;
      dBase       : DB.Object := Data_Base( this );
      Result      : Boolean := False;

      type Action_Type is ( Perform_Insert, Perform_Update );

      To_Do       : Action_Type := Perform_Update;


   begin

      return Result;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Store_Query;

   --|
   --| Delete an Instance from the data base
   --|
   function Delete_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );
   begin
      Enter( debug, Module & ".Load_Query");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Delete_Query;


end DBS.ACDE;

