--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-element.adb,v $
--| Description     : Database element
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--| Functional Description
--| ======================
--| This module contains the basis behaivour for all data base 
--| objects.
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: dbs-element.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

with List;		           use List;
with Interface.List_Element;	   use Interface.List_Element;
use  Interface;

with Simple_List_Element;

package body DBS.Element is

   Version : constant String := "$Id: dbs-element.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "dbs-element.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---
   type Object_State is (
   	Undefined,
	Valid,
	Modified );

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Watch   : List.Object;

         dBase   : DB.Object;  
         Debug   : Debugging_Support.Handle := null;
	 State   : Object_State             := Undefined;
      end record;

   ---=====================================================================---
   ---===           L O C A L   S U P P O R T   T Y P E S 		===---
   ---=====================================================================---

   type Watch_Record is record
         Handler : Watch_Handler := null;
	 Target  : Handle := null; 
      end record;

   package Watch_List_Item is 
   	  new Simple_List_Element( Item_Type => Watch_Record );
   use Watch_List_Item;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object'Class; info : String ) 
                                               return  Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;

   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the initialization
   --| procedure of the extension.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access := null;
   begin
      Enter( debug, Module & ".Initialize" );
      if This.Data /= null then
         raise Usage_Error;
      else
         Data := new Object_Data;
      end if;
      This.Data := Data;
      
      Data.Debug := debug; 
      Initialize( Data.Watch );

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Leave( Debug, "**exception**");
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object'Class ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Finalize( Data.Watch );

      Free( Data );
      this.data := null;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Set the data base handle
   --|
   procedure Data_Base( this : in out Object'Class; dBase : in DB.Object ) is
      Data  : Object_Data_Access := Instance( this, ".Data_Base");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Data_Base" );

      Data.dBase := dBase;

      Leave( Debug, "" );
   exception
      when The_Error : Others =>
         Leave( Debug, "**exception**");
         Error( this, The_Error, ".Data_Base" );
         raise;
   end Data_Base;

   --|
   --| get the database handle
   --|
   function Data_Base( this : in Object'Class ) return DB.Object is
      data : Object_Data_Access := Instance( this, ".Data_Base");
   begin
      return Data.dBase;
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Data_Base");
         raise;
   end Data_Base;

   --|
   --| Add a watch handler to the object 
   --|
   procedure Watch( this    : in out Object'Class; 
                    handler : in Watch_Handler;
		    target  : in Handle ) 
   is
      Data  : Object_Data_Access := Instance( this, ".Watch");
      Debug : Debugging_Support.Handle renames Data.Debug;
      h     : Watch_Record;
      item  : List_Element.Handle := null;
   begin
      Enter( Debug, Module & ".Watch");
      H.Handler := handler;
      H.Target  := target;

      item := Construct( h );
      Append( Data.Watch, item.all );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave(Debug, "** exception **");
         Error( this, The_Error, ".Watch" );
         raise;
   end Watch;

   --|
   --| Return the valid status
   --|
   function Is_Valid( this : in Object'Class ) return Boolean is
      Data : Object_Data_Access := Instance( this, ".Is_Valid");
   begin
      return Data.State = Valid or Data.State = Modified;
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Is_Valid");
         raise;
   end Is_Valid;

   --|
   --| set modified
   --|
   procedure Set_Modified( this : in out Object'Class )  is
      Data : Object_Data_Access := Instance( this, "Set_Modified");
   begin
      Data.State := Modified;
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Is_Valid");
         raise;
   end Set_Modified;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| This procedure informs all handlers about the update ofthe 
   --| database object.
   --|
   procedure Update( this : in out Object'Class ) is
      Data   : Object_Data_Access := Instance( this, ".Update" );
      Debug  : Debugging_Support.Handle renames Data.Debug;

      Item   : List_Element.Handle := null;
      H      : Watch_Record;
   begin
      Enter( Debug, Module & ".Update" );

      if Data.State = Modified then
         if not Store_Query( this ) then
	    raise Usage_Error;
	 end if;
      end if;

      Item := Get( Data.Watch, First );
      while item /= null loop
	 H := Value( Watch_List_Item.Object(Item.all) );

	 if H.Handler /= null then
	    H.handler( This, H.Target  );
	 end if;

         Item := Get( Data.Watch );
      end loop;
 
      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave( Debug, "**exception**");
         Error( this, The_Error, ".Update" );
         raise;
   end Update;

   --|
   --| Load the object instance data from the data base
   --|						   
   procedure Load( this : in out Object'Class ) is
      Data  : Object_Data_Access := Instance( this, ".Load" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Load");

      if not Load_Query( this ) then
         Data.State := Undefined;
      else
         Data.State := Valid;
      end if;

      Update( This );

      Leave( Debug, "" );
   exception
      when The_Error : Others =>
         Leave( Debug, "**exception**");
         Error( this, The_Error, ".Load" );
         raise;
   end Load;

   --|
   --| Delete the object from the database-
   --|
   procedure Delete( this : in out Object'Class ) is
      Data  : Object_Data_Access := Instance( this, ".Delete" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Delete");

      if Delete_Query( this ) then
         Data.State := Undefined;
         Update( This );
      else
         Data.State := Valid;
      end if;
 
      Leave( Debug, "" );
   exception
      when The_Error : Others =>
         Leave( Debug, "**exception**");
         Error( this, The_Error, ".Load" );
         raise;
   end Delete;


end DBS.Element;

