--|  
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-element.ads,v $
--| Description     : Database element
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--| Functional Description
--| ======================
--|
--| This package supplies the basic infrastructure for database elements.
--| The typical life cycle of such an object is:
--|
--|   1. create an instance
--|   2. Add the object to a database
--|   3. load the instance from the database
--|   4. Use the data / update 
--|   5. Update the data in the database
--|   .. go back to 4
--|   6. If desired the object is deleted by deleting it.
--|
--| Step 1 - Instance creation
--|
--|      item : User.Object( dbase );
--| 
--|
--| Step 2 - Add object to database
--|   This step is done by either adding the item to a data base in
--|   step 1 as discriminator or simply by setting the data base
--|   pointer of the object: 
--|
--|      Data_Base( Item, dbase );
--|
--| Step 3 - Load instance data from data base
--|
--|   First of all key attributes of the object are set by means 
--|   of normal attribute procedures, i.e:
--|
--|      Name( item, "erdmann" );
--|
--|   After all parameters have been set the actual item is restored
--|   from the data base.
--|
--|      Load( item );
--|
--|   In order to verify that the object has been loaded correctly
--|   the following function may be used:
--|
--|      if not Is_Valid( item ) then 
--|	       ... error handling ..
--|      end if;
--|
--| Step 4 - data update
--|
--|   Data may be changed by means of attribute functions and 
--|   operations. After completion of the update always the 
--|   the update procedure should be applied.
--|
--| Step 5 - Update in data base
--|  
--|   The object may be marked as modified. Later in the program
--|   execution the update procedure may be called. Upon this
--|   call the object will be stored in the data base and 
--|   all object in the watch list are informed about the 
--|   change.
--| 
--| Step 6 - Delete object
--|
--|   The object is deleted from the database by means of the 
--|   Delete Method, .i.e:
--|   
--|     Delete( User_Object );
--|
--| Component Data
--| ==============
--|
--| Data_Base  - The database where the object is assigned to
--| Valid      - Is_Valid indicates that the object is valid
--|
--| Operations on the component data
--| ================================
--|
--| Extension
--| =========
--|
--| Restrictions
--| ============
--| Tasking: yes/no
--| Y2K    :
--|
--| The idea of this package is not to implement a general persistency
--| concept, but to implement a simple protocol for data base access
--| on object level.
--|
--| History
--| =======
--| $Log: dbs-element.ads,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--| 
--|
with ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

package DBS.Element is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is abstract tagged private;
   type Handle is access Object'Class;

   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null);

   procedure Finalize( this : in out Object'Class );

   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---

   type Watch_Handler is access procedure( source : in Object'Class;
                                           target : in Handle );
   ---------------------------------------------------------------------------
   --| Description    :
   --|    Install a watch handler for the current object. In case the 
   --|    the current object is updated, the handler is called with a
   --|    pointer to a target db element.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --|    None
   --| Exceptions     :
   --|    Any
   --| Note           :
   --|    This feature may be used to map dependencies between object,
   --|    which means if the source object is changed, the target object
   --|    has to be reevaluated again. 
   ---------------------------------------------------------------------------
   procedure Watch( this    : in out Object'Class; 
                    handler : in Watch_Handler;
		    target  : in Handle );

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Return the data base handle, where the object is assigined to.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --|    No state change
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Data_Base( This : in Object'Class ) return DB.Object;

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Set the database handle.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --|    No state change 
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Data_Base( This : in out Object'Class;  dBase : in DB.Object);

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Return true if the object is valid
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --|    No state change 
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Is_Valid( This : in Object'Class ) return Boolean;
   
   ---------------------------------------------------------------------------
   --| Description    :
   --|    Mark the object as modified. This call will normally be used
   --|    by extension of this class if some data is changed. The next
   --|    update call will write back the object to the data base.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --|    No state change 
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Set_Modified( this : in out Object'Class );
 
   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    :
   --|    This method indicates that the contents of the object has been
   --|    changed.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Update( This : in out Object'Class );

   ---------------------------------------------------------------------------
   --| Description    :
   --|    This method initiates a load from the data base. This method
   --|    invokes the Update procedure upon successful loading.
   --|    In case of an problem loading the data the object will be 
   --|    marked as invalid.
   --|
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --|    2. All required attributes are set.
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Load( this : in out Object'Class );
   ---------------------------------------------------------------------------
   --| Description    :
   --|	   The object will be deleted from the data base by execution of
   --|     the Delete_Query procedure.
   --| Preconditions  :
   --|    1. Instance has been initialized.
   --|    2. All required key attributes are set.
   --| Postconditions :
   --|    Object state is invalid if the delete was successful.
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Delete( this : in out Object'Class );

   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---

   function Load_Query( this : in Object ) return Boolean is abstract;

   function Store_Query( this : in Object ) return Boolean is abstract;

   function Delete_Query( this : in Object ) return Boolean is abstract;
   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is  abstract tagged record
         data  : Object_Data_Access := null;
      end record; 

end DBS.Element;


