--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-project.adb,v $
--| Description     : project class.
--| Author          : Michael Erdmann
--| Created On      : 4.1.2000
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.1 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This module contains the project class. The object maintains the 
--| complete view of a project.
--|
--| Project class is an implementation of the dbelement class.
--|
--| Restrictions
--| ============
--| None
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: dbs-project.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;

--- Ascl
with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;			   use ASCL.DB;
use  ASCL;

--- Database objects
with DBS.Element;	            use DBS.Element;
with DBS.T_Projectlist;		    use DBS.T_Projectlist;
with DBS.SQL;			    use DBS.SQL;

--- Common support procedures
with List;			    use List;
with Interface.List_Element;        use Interface.List_Element;
with Simple_List_Element;
use  Interface;

package body DBS.Project is

   Version : constant String := "$Id: dbs-project.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "dbs-project.adb";

   ---====================================================================---
   ---===                 C L A S S    D A T A			       ===---
   ---====================================================================---
   User_Data_Base : DB.Object;
   User_Debug     : Debugging_Support.Handle := null;

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Name           : Unbounded_String := To_Unbounded_String("");
	 Owner		: Unbounded_String := To_Unbounded_String("");
	 Creation_Date  : Time; 
         Modify_Date    : Time; 

         Debug   : Debugging_Support.Handle := null;
      end record;
   			
   ---======================================================================---
   ---===            L O C A L  S U P P O R T   T Y P E S                ===---
   ---======================================================================---
   
   package Project_List_Item_Pkg is 
            new Simple_List_Element( Item_Type => Unbounded_String ); 
   use Project_List_Item_Pkg;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object; info : String ) return  Object_Data_Access is
   begin
      if this.Data = null then
         raise ASCL.Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;

   --|
   --| Format a date string 
   --|
   function Date( atTime : in Time ) return Unbounded_String is
      YY : Year_Number;
      MM : Month_Number;
      DD : Day_Number;
      SS : Day_Duration;
   begin
      Split( atTime, YY, MM, DD, SS );
      return TO_Unbounded_String( 
                Year_Number'Image(YY) & "-" & 
                Month_Number'Image(MM) & "-" &
	        Day_Number'Image(DD));
   end Date;
   			
   ---======================================================================---
   ---===                 C L A S S   I N T E R F A C E                  ===---
   ---======================================================================---

   procedure Initialize( dBase : in DB.Object;
   			 debug : in Debugging_Support.Handle := null ) is
   begin
      User_Data_Base := dBase;
      User_Debug     := debug;

      SQL.Initialize( debug );
   end Initialize;

   procedure Finalize is
   begin
      null;
   end Finalize;
   			
   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the Initialization
   --| procedure for the extension and any other object data
   --| component.
   --|
   --| If the data for the class data is not yet set 
   --|
   procedure Initialize( this  : in out Object;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise ASCL.Usage_Error;
      end if;

      DBS.Element.Initialize( this, debug );

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug; 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

      DBS.Element.Finalize( this );

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| This function returns the first name part of the real name
   --|
   function Name( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Name");
   begin
      return To_String(data.Name);
   end Name;

   --|
   --| This function returns the first name part of the real name
   --|
   function Owner( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Owner");
   begin
      return To_String(data.Owner);
   end Owner;

   --|
   --| This function returns the first name part of the real name
   --|
   function Creation_Date( this : in Object ) return Time is
      data : Object_Data_Access := Instance( this, ".Creation_Date");
   begin
      return Data.Creation_Date;
   end Creation_Date;

   --|
   --| This function returns the first name part of the real name
   --|
   function Modification_Date( this : in Object ) return Time is
      data : Object_Data_Access := Instance( this, ".Login_Name");
   begin
      return Data.Modify_Date;
   end Modification_Date;

   --|
   --| Set some attribute
   --|
   procedure Name( this : in out Object; name : in Unbounded_String ) is
      Data  : Object_Data_Access := Instance( this, ".Name");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Name=" & To_String(name) );

      Data.Name :=  name;
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Name" );
         raise;
   end Name;

   procedure Owner( this : in out Object; name : in Unbounded_String ) is
      Data  : Object_Data_Access := Instance( this, ".Owner");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Owner=" & To_String(name) );

      Data.Owner :=  name;
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Owner" );
         raise;
   end Owner;

   --|
   --| Set some attribute
   --|
   procedure Creation_Date( this : in out Object; atTime : Time ) is
      Data  : Object_Data_Access := Instance( this, ".Creation_Date");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Creation_Date" );

      Data.Creation_Date := atTime;
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Creation_Date" );
         raise;
   end Creation_Date;

   --|
   --| Set some attribute
   --|
   procedure Modification_Date( this : in out Object; atTime : Time ) is
      Data  : Object_Data_Access := Instance( this, ".Modifcation_Date");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Modification_Date" );

      Data.Modify_Date := atTime;
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Modification_Date" );
         raise;
   end Modification_Date;

   ---=====================================================================---
   ---===                C L A S S   M E T H O D S                      ===---
   ---=====================================================================---
   function All_Projects return List.Object is
      qID    : DB.Query_ID;
      dBase  : DB.Object renames User_Data_Base;
      Result : List.Object;
      Name   : Unbounded_String    := Null_Unbounded_String;
      Item   : List_Element.Handle := null;
   begin
      Enter( User_Debug, Module & ".All_Users" );

      Initialize( Result, User_Debug );

      qID := DB.Query( dBase, SQL_Select( From => T_Project_List ) );
      if DB.Nbr_Of_Rows( dBase, qID ) = 0 then
         DB.Drop_Query( dBase, qID );
	 return Result;
      end if;

      while true loop
         begin
            Name := String_Field( dBase, qID, D_Project_Name );
	    Trace( User_Debug, "name = " & To_String( Name ));
	    Item := Construct( name );
	    Append( Result, item.all );

            DB.Next( dBase, qID );

	 exception
	    when DB.Field_Parse_Error =>
	       Trace( User_Debug, "Parser error" );
	       DB.Next( dBase, qID );
	    when DB.No_More_Rows =>    
	       DB.Drop_Query( dBase, qID );
	       exit;
	    when Others =>
	       raise;
	 end;
      end loop;

      DB.Drop_Query( dBase, qID );

      Leave( User_Debug, "" );
      return Result;

   end All_Projects;

   --|
   --| Get the next user from the list of all users.
   --|
   function Get( users : in List.Object; start : Boolean := False ) 
                                                    return String is
      Item : List_Element.Handle;
      Name : Unbounded_String;
   begin
      Enter( User_Debug, Module & ".Get" );
      if start then
         Item := List.Get( users, List.First );
      else
         Item := List.Get( users );
      end if;

      if Item = null then
          return "";  
      end if;        
  
      name := Value( Project_List_Item_Pkg.Object(item.all) );
      
      Leave( User_Debug, "Name = " & To_String( Name ) ); 
      return To_String( name );
   end Get;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Load the user data for a given login name into the 
   --| object instance.
   --|
   function Load_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );   
   begin
      Enter( debug, Module & ".Load_Query");

      begin
         qId := DB.Query( 
	            dBase,
	            SQL_Select( 
		       from  => T_Project_List,
		       where => SQL_Colum(D_Project_Name, Data.Name)
		    )
		 ); 

	 Data.Name          := DB.String_Field(dBase, qID, D_Project_Name );
	 Data.Owner	    := DB.String_Field(dBase, qID, D_Owner_Name   );
	 Data.Creation_Date := DB.Date_Field(dBase,   qId, D_Creation_Date);
	 Data.Modify_date   := DB.Date_Field(dBase,   qId, D_Modification_Date );

	 DB.Drop_Query( dBase, qID );
      exception
         when Others =>
	    raise;		         
      end ;

      Leave( debug, "");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Load_Query;

   --|
   --| Store the object information in the data base. If update fails
   --| perform an insert. If this fails as well raise an exception.
   --|
   function Store_Query ( this : in Object) return Boolean is
      Data        : Object_Data_Access := Instance( this, ".Store_Query");
      Debug       : Debugging_Support.Handle renames data.Debug;
      qID         : DB.Query_ID;
      dBase       : DB.Object := Data_Base( this );
      Result      : Boolean := False;

      type Action_Type is ( Perform_Insert, Perform_Update );

      To_Do       : Action_Type := Perform_Update;

      Data_Record : Unbounded_String := To_Unbounded_String("");

      -- create an update string 
      procedure Arg( domain : in String; value : in Unbounded_String ) is
      begin
         if Data_Record = "" then
            Data_Record := TO_Unbounded_String(SQL_Colum( domain, value ));
	 else
	    Data_Record := Data_Record & "," & SQL_Colum( domain, value ); 
	 end if;
      end Arg;

      -- create a project tuple. The sequence of fields has to be
      -- in line with ../DB/T_Project.sql.
      function Tuple return Unbounded_String is
         Result : Unbounded_String;
      begin
         Result := To_Unbounded_String( SQL_String( Data.Name ) );
         Result := Result & "," & SQL_String( Data.Owner );	 
	 Result := Result & "," & SQL_String( Date( Data.Creation_Date ) );
	 Result := Result & "," & SQL_String( Date( Data.Modify_Date ) );
	 return Result;
      exception 
         when Others =>
	    Trace(Debug, "** exception in Tuple catched **");
	    return Result;
      end Tuple;

   begin
      Enter( debug, Module & ".Store_Query");

      begin
         qId := DB.Query( dBase, 
	           SQL_Select( from => T_Project_List,
	                       where => SQL_Colum(D_Project_Name, Data.Name) 
	           ) 
		);
	 if Nbr_Of_Rows( dBase, qID ) > 0  then
	    To_Do := Perform_Update;
	 else
	    To_Do := Perform_Insert;
	 end if;      
	 Drop_Query( dBase, qId );	
      exception
         when DB.No_Query_Result =>
	    To_Do := Perform_Insert;
	    Trace(Debug, "Exception - No_Query_Result - catched");
	 when Others => 
	    raise; 
      end;
   	        
     
      begin
         case To_Do is
	    when Perform_Update => 
               Trace( Debug, "Store_Query - Perform Update ");
      	       Arg( D_Project_Name,      Data.Name );
	       Arg( D_Owner_Name,        Data.Owner );
	       Arg( D_Creation_Date,     Date(Data.Creation_Date) );
	       Arg( D_Modification_date, Date(data.Modify_date) );

               qId := DB.Query( 
	                  dBase, 
	 	          SQL_Update( 
		             update => T_Project_List,
		             colum  => To_String(Data_Record),
		             where  => SQL_Colum(D_Project_Name, Data.Name)
		          )
		      );
	    when Perform_Insert =>
               Trace( Debug, "Store_Query - Perform Insert ");
               qId := DB.Query( 
		         dBase, 
	 	         SQL_Insert( 
		             into   => T_Project_List,
		             value  => Tuple
		         )
		      );
	       Trace(Debug, "Insert complete");
         end case;

         DB.Drop_Query( dBase, qID );   
	 Result := True;

      exception
         when DB.No_Query_Result =>
	    Result := False;
         when Others =>
	    raise;		         
      end ;
  
      Leave( debug, "");
      return Result;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Store_Query;

   --|
   --| Delete an Instance from the data base
   --|
   function Delete_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );
   begin
      Enter( debug, Module & ".Load_Query");

      begin
         qId := DB.Query( 
	            dBase,
	            SQL_Delete( 
		       from  => T_Project_List,
		       where => 
		           To_Unbounded_String(SQL_Colum(D_Project_Name, Data.Name))
		    )
		 ); 
	 DB.Drop_Query( dBase, qID );
      exception
         when Others =>
	    raise;		         
      end ;

      Leave( debug, "");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Delete_Query;


   --|
   --| Store the object information in the data base. If update fails
   --| perform an insert. If this fails as well raise an exception.
   --|
   function Exists ( this : in Object; name : in String ) return Boolean is
      Data        : Object_Data_Access := Instance( this, ".Store_Query");
      Debug       : Debugging_Support.Handle renames data.Debug;
      qID         : DB.Query_ID;
      dBase       : DB.Object := Data_Base( this );
      Result      : Boolean := False;
   begin
      Enter( Debug, Module & ".Exists");

      qId := DB.Query( 
                dBase, 
	        SQL_Select( from => T_Project_List,
	                    where => D_Project_Name & "='" & Name & "'" 
	        ) 
             );

      if Nbr_Of_Rows( dBase, qID ) > 0  then
         Result := True;
      end if;      
      Drop_Query( dBase, qId );	

      Leave(Debug,"");
      return Result;

   exception
      when DB.No_Query_Result =>
         Leave(Debug, "");
         return False;
      when others =>
         Leave(Debug,"** exception **");
         raise;
   end Exists;

end DBS.Project;

