--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-user.adb,v $
--| Description     : User object class.
--| Author          : Michael Erdmann
--| Created On      : 13.12.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.1 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This component maintains user object. A user object contains
--| all data for a certain user. It is implemented by the  
--| Object_Data structure visible only in this module.
--|
--| User class is an implementation of the dbelement class.
--|
--| Restrictions
--| ============
--| None
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: dbs-user.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;

--- Ascl
with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;			   use ASCL.DB;
use  ASCL;

--- Database objects
with DBS.Element;	            use DBS.Element;
with DBS.T_Userlist;		    use DBS.T_Userlist;
with DBS.SQL;			    use DBS.SQL;
use  DBS;

--- Common support procedures
with List;			    use List;
with Interface.List_Element;        use Interface.List_Element;
with Simple_List_Element;

use  Interface;

package body DBS.User is

   Version : constant String := "$Id: dbs-user.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "user.adb";

   ---====================================================================---
   ---===                 C L A S S    D A T A			       ===---
   ---====================================================================---
   User_Data_Base : DB.Object;
   User_Debug     : Debugging_Support.Handle := null;

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         First_Name : Unbounded_String;
	 Last_Name  : Unbounded_String;
	 Login      : Unbounded_String;

	 Org_Unit   : Unbounded_String;
	 Phone      : Unbounded_String;
	 Email      : Unbounded_String;
	 Location   : Unbounded_String;

         Debug   : Debugging_Support.Handle := null;
      end record;
   			
   ---======================================================================---
   ---===            L O C A L  S U P P O R T   T Y P E S                ===---
   ---======================================================================---
   
   package User_List_Item_Pkg is 
            new Simple_List_Element( Item_Type => Unbounded_String ); 
   use User_List_Item_Pkg;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object; info : String ) return  Object_Data_Access is
   begin
      if this.Data = null then
         raise ASCL.Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;
   			
   ---======================================================================---
   ---===                 C L A S S   I N T E R F A C E                  ===---
   ---======================================================================---

   procedure Initialize( dBase : in DB.Object;
   			 debug : in Debugging_Support.Handle := null ) is
   begin
      User_Data_Base := dBase;
      User_Debug     := debug;

      SQL.Initialize( debug );
   end Initialize;

   procedure Finalize is
   begin
      null;
   end Finalize;
   			
   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the Initialization
   --| procedure for the extension and any other object data
   --| component.
   --|
   --| If the data for the class data is not yet set 
   --|
   procedure Initialize( this  : in out Object;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise ASCL.Usage_Error;
      end if;

      DBS.Element.Initialize( this, debug );

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug; 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extention.
   --|
   procedure Finalize( this : in out Object ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

      DBS.Element.Finalize( this );

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| This function returns the first name part of the real name
   --|
   function Real_First_Name( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Real_First_Name");
   begin
      return To_String( data.First_Name );
   end Real_First_Name;


   --|
   --| This function returns the first name part of the real name
   --|
   function Real_Last_Name( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, "Real_Last_Name");
   begin
      return To_String( data.Last_Name );
   end Real_Last_Name;


   --|
   --| This function returns the first name part of the real name
   --|
   function Login_Name( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Login_Name");
   begin
      return To_String( data.Login );
   end Login_Name;

   --|
   --| This function returns the first name part of the real name
   --|
   function Org_Unit( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Org_Unit");
   begin
      return To_String( data.Org_Unit );
   end Org_Unit;

   --|
   --| This function returns the first name part of the real name
   --|
   function Phone( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Phone");
   begin
      return To_String( data.Phone );
   end Phone;

   --|
   --| This function returns the first name part of the real name
   --|
   function Email( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Email");
   begin
      return To_String( data.Email );
   end Email;

   --|
   --| This function returns the first name part of the real name
   --|
   function Location( this : in Object ) return String is
      data : Object_Data_Access := Instance( this, ".Location");
   begin
      return To_String( data.Location );
   end Location;

   --|
   --| Set some attribute
   --|
   procedure Real_First_Name( this : in out Object; name : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Real_First_Name");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Real_First_Name=" & name );

      Data.First_Name := To_Unbounded_String( name );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Real_First_Name" );
         raise;
   end Real_First_Name;

   --|
   --| Set some attribute
   --|
   procedure Real_Last_Name( this : in out Object; name : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Real_Last_Name");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Real_Last_Name=" & name );

      Data.Last_Name := To_Unbounded_String( name );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Real_Last_Name" );
         raise;
   end Real_Last_Name;

   --|
   --| Set some attribute
   --|
   procedure Login_Name( this : in out Object; name : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Login_Name");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Login_Name=" & name );

      Data.Login := To_Unbounded_String( name );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Login_Name" );
         raise;
   end Login_Name;

   --|
   --| Set some attribute
   --|
   procedure Org_Unit( this : in out Object; unit : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Org_Unit");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Org_Unit=" & unit );

      Data.Org_Unit := To_Unbounded_String( unit );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Org_Unit" );
         raise;
   end Org_Unit;

   --|
   --| Set some attribute
   --|
   procedure Phone( this : in out Object; numb : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Phone");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Phone=" & numb );

      Data.Phone := To_Unbounded_String( numb );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Phone" );
         raise;
   end Phone;

   --|
   --| Set some attribute
   --|
   procedure Email( this : in out Object; addr : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Email");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, "Email=" & addr );

      Data.Email := To_Unbounded_String( addr );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Email" );
         raise;
   end Email;

   --|
   --| Set some attribute
   --|
   procedure Location( this : in out Object; addr : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Email");
      Debug : Debugging_Support.Handle renames data.Debug;
   begin
      Enter( Debug, Module & ".Location=" & addr );

      Data.Location := To_Unbounded_String( addr );
      Set_Modified( this );

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Location" );
         raise;
   end Location;

   ---=====================================================================---
   ---===                C L A S S   M E T H O D S                      ===---
   ---=====================================================================---
   function All_Users return List.Object is
      qID    : DB.Query_ID;
      dBase  : DB.Object renames User_Data_Base;
      Result : List.Object;
      Name   : Unbounded_String    := Null_Unbounded_String;
      Item   : List_Element.Handle := null;
   begin
      Enter( User_Debug, Module & ".All_Users" );

      Initialize( Result, User_Debug );

      qID := DB.Query( dBase,
                      "select " & D_User_Name & " from " & T_User_List   );
      if DB.Nbr_Of_Rows( dBase, qID ) = 0 then
         DB.Drop_Query( dBase, qID );
	 return Result;
      end if;

      while true loop
         begin
            Name := String_Field( dBase, qID, D_User_Name );
	    Trace( User_Debug, "name = " & To_String( Name ));
	    Item := Construct( name );
	    Append( Result, item.all );

            DB.Next( dBase, qID );

	 exception
	    when DB.Field_Parse_Error =>
	       Trace( User_Debug, "Parser error" );
	       DB.Next( dBase, qID );
	    when DB.No_More_Rows =>    
	       DB.Drop_Query( dBase, qID );
	       exit;
	    when Others =>
	       raise;
	 end;
      end loop;

      DB.Drop_Query( dBase, qID );

      Leave( User_Debug, "" );
      return Result;

   end All_Users;

   --|
   --| Get the next user from the list of all users.
   --|
   function Get( users : in List.Object; start : Boolean := False ) 
                                                    return String is
      Item : List_Element.Handle;
      Name : Unbounded_String;
   begin
      Enter( User_Debug, Module & ".Get" );
      if start then
         Item := List.Get( users, List.First );
      else
         Item := List.Get( users );
      end if;

      if Item = null then
          return "";  
      end if;        
  
      name := Value( User_List_Item_Pkg.Object(item.all) );
      
      Leave( User_Debug, "Name = " & To_String( Name ) ); 
      return To_String( name );
   end Get;
   

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Load the user data for a given login name into the 
   --| object instance.
   --|
   function Load_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );
   
      function cpField ( name : String ) return Unbounded_String is
      begin
         return DB.String_Field( Data_Base( this ), qId, name ) ;
      end cpField;

   begin
      Enter( debug, Module & ".Load_Query");

      begin
         qId := DB.Query( 
	            Data_Base(This),
	            SQL_Select( 
		       from  => T_User_List,
		       where => SQL_Colum(D_User_Name, Data.login)
		    )
		 ); 
	 Data.login     := cpField( D_User_Name );
	 Data.First_Name:= cpField( D_First_Name );
	 Data.Last_Name := cpField( D_Last_Name );
	 Data.Org_Unit  := cpField( D_Org_Unit );
	 Data.Phone     := cpField( D_Phone );
	 Data.Email	:= cpField( D_Email );
	 Data.Location  := cpField( D_Location );

	 DB.Drop_Query( dBase, qID );
      exception
         when Others =>
	    raise;		         
      end ;

      Leave( debug, "");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Load_Query;

   --|
   --| Store the object information in the data base. If update fails
   --| perform an insert. If this fails as well raise an exception.
   --|
   function Store_Query ( this : in Object) return Boolean is
      Data        : Object_Data_Access := Instance( this, ".Store_Query");
      Debug       : Debugging_Support.Handle renames data.Debug;
      qID         : DB.Query_ID;
      dBase       : DB.Object := Data_Base( this );
      Result      : Boolean := False;

      type Action_Type is ( Perform_Insert, Perform_Update );

      To_Do       : Action_Type := Perform_Update;

      Data_Record : Unbounded_String := To_Unbounded_String("");

      -- create an update string 
      procedure Arg( domain : in String; 
                     value  : in Unbounded_String ) is
      begin
         if Data_Record = "" then
            Data_Record := TO_Unbounded_String(SQL_Colum( domain, value ));
	 else
	    Data_Record := Data_Record & "," & SQL_Colum( domain, value ); 
	 end if;
      end Arg;

      -- create a USERLIST tuple. The sequence of fields has to be
      -- in line with ../DB/T_Userlist.sql.
      function User_Tuple return Unbounded_String is
         Result : Unbounded_String := To_Unbounded_String("");
      begin
         Result := Result & SQL_String( Data.Login );
         Result := Result & "," & SQL_String( To_Unbounded_String("acde.") );
         Result := Result & "," & SQL_String( Data.First_Name );
         Result := Result & "," & SQL_String( Data.Last_Name );
         Result := Result & "," & SQL_String( Data.Org_Unit );
         Result := Result & "," & SQL_String( Data.Phone );
         Result := Result & "," & SQL_String( Data.Location );
         Result := Result & "," & SQL_String( Data.Email );
	 Result := Result & "," & SQL_String( To_Unbounded_String("0") );

	 return Result;
      end User_Tuple;
   begin
      Enter( debug, Module & ".Store_Query");

      begin
         qId := DB.Query( dBase, 
	           SQL_Select( from => T_User_List,
	                       where => SQL_Colum(D_User_Name, Data.Login) 
	           ) 
		);
	 if Nbr_Of_Rows( dBase, qID ) > 0  then
	    To_Do := Perform_Update;
	 else
	    To_Do := Perform_Insert;
	 end if;      
	 Drop_Query( dBase, qId );	
      exception
         when DB.No_Query_Result =>
	    To_Do := Perform_Insert;
	 when Others => 
	    raise; 
      end;
   	        

      begin
         case To_Do is
	    when Perform_Update => 
      	       Arg( D_User_Name,  Data.Login );
      	       Arg( D_First_Name, Data.First_Name );
      	       Arg( D_Last_Name,  Data.Last_Name );
      	       Arg( D_Org_Unit,   Data.Org_Unit );
      	       Arg( D_Phone,      Data.Phone );
      	       Arg( D_Email,      Data.Email );
      	       Arg( D_Location,   Data.Location);

               qId := DB.Query( 
	                  dBase, 
	 	          SQL_Update( 
		                 update => T_User_List,
		                 colum  => To_String(Data_Record),
		                 where  => SQL_Colum(D_User_Name, Data.Login)
		          )
		      );
	    when Perform_Insert =>
               qId := DB.Query( 
		         dBase, 
	 	         SQL_Insert( 
		            into   => T_User_List,
		            value  => User_Tuple
		         )
		      );
         end case;

         DB.Drop_Query( dBase, qID );   
	 Result := True;

      exception
         when DB.No_Query_Result =>
	    Result := False;
         when Others =>
	    raise;		         
      end ;
  
      Leave( debug, "");
      return Result;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Store_Query;

   --|
   --| Delete an Instance from the data base
   --|
   function Delete_Query( this : in Object) return Boolean is
      Data  : Object_Data_Access := Instance( this, ".Load_Query");
      Debug : Debugging_Support.Handle renames data.Debug;
      qID   : DB.Query_ID;
      dBase : DB.Object := Data_Base( this );
   begin
      Enter( debug, Module & ".Load_Query");

      begin
         qId := DB.Query( 
	            Data_Base(This),
	            SQL_Delete( 
		       from  => T_User_List,
		       where => 
		           To_Unbounded_String(SQL_Colum(D_User_Name, Data.login))
		    )	 
		 ); 
	 DB.Drop_Query( dBase, qID );
      exception
         when Others =>
	    raise;		         
      end ;

      Leave( debug, "");
      return True;

   exception
      when The_Error : Others =>
         Leave( debug, "** exception **");
         return False;
   end Delete_Query;

   --|
   --| Store the object information in the data base. If update fails
   --| perform an insert. If this fails as well raise an exception.
   --|
   function Exists ( this : in Object; name : in String ) return Boolean is
      Data        : Object_Data_Access := Instance( this, ".Store_Query");
      Debug       : Debugging_Support.Handle renames data.Debug;
      qID         : DB.Query_ID;
      dBase       : DB.Object := Data_Base( this );
      Result      : Boolean := False;
   begin
      Enter( Debug, Module & ".Exists");

      qId := DB.Query( 
                dBase, 
	        SQL_Select( from => T_User_List,
	                    where => D_User_Name & "='" & Name & "'" 
	        ) 
             );

      if Nbr_Of_Rows( dBase, qID ) > 0  then
         Result := True;
      end if;      
      Drop_Query( dBase, qId );	

      Leave(Debug,"");
      return Result;

   exception
      when DB.No_Query_Result =>
         Leave(Debug, "");
         return False;
      when others =>
         Leave(Debug,"** exception **");
         raise;
   end Exists;


end DBS.User;

