--|  
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dbs-user.ads,v $
--| Description     : user class
--| Author          : Michael Erdmann
--| Created On      : 13.12.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--|
--| Functional Description
--| ======================
--| This module implements the data base interface to the user object
--| and the class as well. 
--|
--| Component Data
--| ==============
--|
--| Real_First_Name   - First part of the real world name
--| Real_Last_Name    - Last name of the real world name
--| Login_Name        - Login name of the user
--| Org_Unit          - Organisation unit
--| Phone             - Phone number
--| Location	      - Location
--| Mail_Address      - Mail Address of the user
--|
--|
--| Class Data
--| ==========
--|
--| Operations on the component/class data
--| ======================================
--|
--| Store   	      - Store an instance in the data base
--| Load              - Load an instance from the data base
--| 
--| All_Users	      - Returns the list of all users
--| Get		      - Get the next user from the list 
--|
--| Extension
--| =========
--|
--| Restrictions
--| ============
--| Tasking: yes/no
--| Y2K    :
--|
--|
--| History
--| =======
--| $Log: dbs-user.ads,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--| Revision 1.7  2000/01/16 16:03:58  ACDE
--| Release Version V8_1test
--|
--|
with ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

with DBS.Element;   
use  DBS;

with List;	

package DBS.User is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is new Element.Object with private;
   type Handle is access Object'Class;

   procedure Initialize( this  : in out Object;
                         debug : in Debugging_Support.Handle := null);
   procedure Finalize( this : in out Object );

   procedure Initialize( dBase : DB.Object;
   			 debug : Debugging_Support.Handle := null );

   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---
   function  Real_Last_Name ( this : in Object ) return String;
   function  Real_First_Name( this : in Object ) return String;
   function  Login_Name     ( this : in Object ) return String;
   function  Org_Unit       ( this : in Object ) return String;
   function  Phone          ( this : in Object ) return String;
   function  Email          ( this : in Object ) return String;
   function  Location       ( this : in Object ) return String;

   procedure Real_Last_Name ( this : in out Object; name : in String );
   procedure Real_First_Name( this : in out Object; name : in String );
   procedure Login_Name     ( this : in out Object; name : in String );
   procedure Org_Unit       ( this : in out Object; unit : in String );
   procedure Phone          ( this : in out Object; numb : in String );
   procedure Email          ( this : in out Object; addr : in String );
   procedure Location       ( this : in out Object; addr : in String );


   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    :
   --|   The methods check whether an object exists or not.
   --| Preconditions  :
   --|   1. Object Initialized.
   --|   2. Object is assigned to a data base
   --|
   --| Postconditions :
   --|   Results true if the object exists, else it returns false.
   --| Exceptions     :
   --|   None
   --| Note           :
   ---------------------------------------------------------------------------
   function Exists( this : in Object; name : in String ) return Boolean;


   ---=====================================================================---
   ---===                   C L A S S   M E T H O D S			===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Generate a list of all users in the system
   --| Preconditions  :
   --|    1. Package has to be initialized 
   --| Postconditions :
   --|    List contains all users in the system
   --| Exceptions     :
   --| Note           :
   --|    If the list is not needed any more it should be finalized by
   --|    the finalize method from the List package.
   ---------------------------------------------------------------------------
   function All_Users return List.Object;

   ---------------------------------------------------------------------------
   --| Description    :
   --|    Get the next entry in the list of all users in the system as
   --|    a string.  
   --|    If the end of the list is reached, an empty string is returned. 
   --| Preconditions  :
   --|    1. Package has to be initialized 
   --|    2. List has to be returned by the All_Users method.
   --| Postconditions :
   --|    Read pointer is set to the next user name.   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Get( users : List.Object; start : in Boolean := False )
                                                             return String;




   ---------------------------------------------------------------------------
   --| Description    :
   --|   This procedure loads the information for a certain user 
   --|   from the data base.
   --|   The procedure returns true if every thing worked correctly.
   --|
   --|   This procedure is required by the abstract class dbelement.
   --| Preconditions  :
   --|   1. Object Initialized.
   --|   2. Object is assigned to a data base
   --|   3. Login_Name set
   --|
   --| Postconditions :
   --|   User data copied from the data base into the attributes
   --| Exceptions     :
   --|   None
   --| Note           :
   ---------------------------------------------------------------------------
   function Load_Query( this : in Object ) return Boolean;
   ---------------------------------------------------------------------------
   --| Description    :
   --|   Store the object data in the data base.
   --|   This method is required by the abstract class dbelement
   --| Preconditions  :
   --|   1. Object is initialized
   --|   2. Object is assigned to a data base
   --|   3. At least the login_name attribute is set.
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Store_Query( this : in Object ) return Boolean;

   ---------------------------------------------------------------------------
   --| Description    :
   --|   Delete the data behind a user object from the database
   --| Preconditions  :
   --|   1. Object is initialized
   --|   2. Object is assigned to a data base
   --|   3. At least the login_name attribute is set.
   --| Postconditions :
   --|   Item delete
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Delete_Query( this : in Object ) return Boolean;

   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is new Element.Object with record
         data : Object_Data_Access := null;
      end record; 

end DBS.User;






