--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/dlg-admin.adb,v $
--| Description     : Handle all project/user related actions
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.1 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================      
--| This module manages the administration dialog via the CGI Interface.
--|
--| Restrictions
--| ============
--| Database installation
--|		        
--| References			       
--| ==========
--|
--|
--| History
--| =======
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Text_IO;                  use Ada.Text_IO;
with Ada.Strings.Unbounded;        use Ada.Strings.Unbounded;
with Ada.Calendar;		   use Ada.Calendar;
with Unchecked_Deallocation;

with CGI;

--- ASCL Interface
with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

--- CGI base process components
with STD.Process;  	           use STD.Process;
with STD.CGI_Debug; 
use  STD;
with Environement;                 use Environement; 

--- Common component
with HTML;			   use HTML;   

--- Database related components 
with DBS.Element;		   use DBS.Element;
with DBS.User;			   use DBS.User;
with DBS.SQL;
with DBS.T_Projectlist;		   use DBS.T_Projectlist;
with DBS.Project;		   use DBS.Project;
use  DBS;

package body DLG.Admin is

   Version : constant String := "$Id: dlg-admin.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "dgl-admin.adb";

   --- External component names
   P_Next_Action    : constant String := "NextAction";
    
   P_Project_Name   : constant String := "ProjectName";
   P_New_Login      : constant String := "NewLogin";
   P_Owner_Name     : constant String := "Owner";
   P_Org_Unit       : constant String := "Org";
   P_Phone          : constant String := "Phone";
   P_Login_Name     : constant String := "LoginName";
   P_Real_Name      : constant String := "realname";
   P_Real_First_Name: constant String := "realfirst";
   P_User_Action    : constant String := "UserAction";
   P_Location       : constant String := "Location";	  
   P_Email          : constant String := "Email";
   --- Constants which are derived from the input forms
   V_Open_Project   : constant String := "OpenProject";
   V_Maint_Project  : constant String := "MaintProject";
   V_Maint_User     : constant String := "MaintUser";
   V_Maint_Role     : constant String := "MaintRole";
   V_Maint_System   : constant String := "MaintSystem";

   V_Create_User    : constant String := "CreateUser";
   V_Delete_User    : constant String := "DeleteUser";
   V_Show_User      : constant String := "ShowUser";

   V_Update_Project : constant String := "UpdateProject";
   V_Delete_Project : constant String := "DeleteProject";
   V_Show_Project   : constant String := "ShowProject";


   type Action_Handler is access procedure ( this : in out Object );

   type Action_Record is record
         name    : Unbounded_String;
         handler : Action_Handler := null;
      end record;

   type Commands is array( Positive range <> ) of Action_Record;

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Transaction : Positive                 := 1;
	 Count       : Integer                  := 0;
	 Debug       : Debugging_Support.Handle := null;
 	 Next_Page   : Customized_Page_Handler  := null;
	 Next_Title  : Unbounded_String		:= Null_Unbounded_String;
	 Error_Text  : Unbounded_String         := Null_Unbounded_String;
	 User_Obj    : User.Object;
	 Project_Obj : Project.Object;
      end record;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occurred.
   --|	     
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Retrieve the object instance data 
   --|
   function Instance( this : Object; info : String ) return Object_Data_Access is
   begin
      if this.data = null then
         raise Not_Initialized;
      end if;
      return this.Data;
   exception
      when The_Error : Others =>
         Error( this, The_Error, Module & ".Instance for " & info );
	 raise; 
   end Instance;

   --|
   --| Generate a text input field, where the descriptive text and 
   --| the input field are in different colums of a table. Both
   --| elements are stored in one row.
   --|
   procedure Field( fieldname : String; 
                    param     : String; 
		    default   : String; 
		    length    : Integer := 0;
		    aux       : String := "" ) is
   begin
      HTML.TR;
         HTML.TD( aux ); 
	    HTML.Print( fieldname ); 
	 HTML.TD_End;

         HTML.TD;
            if length /= 0 then
               Text( Name  => param, 
	             Value => default, 
		     Size  => length ); 
            else
	       Text( Name  => param, 
	             Value => default); 	    
            end if;
         HTML.TD_End;
      HTML.TR_End;
   end Field;

   --|
   --| Place a radio button and a descriptive text in two
   --| different colums of a table and the same row. 
   --|
   procedure Radio( fieldname : String; param : String; aux : String := "" ) is
   begin
      HTML.TR;
         HTML.TD( aux ); HTML.Print(fieldname); HTML.TD_End;
         HTML.TD( aux );
            Radio_Button( Name => P_Next_Action, Value => param );
	 HTML.TD_End;
      HTML.TR_End;
   end Radio;

   --|
   --| This procedure displays the error text in the the 
   --| output form assuming the form is contained within a 
   --| table.
   --|
   procedure Handle_Error_Text( this : in out Process.Object'Class ) is
      Data : Object_Data_Access renames Object(this).Data;
   begin     
      if Data.Error_Text /= Null_Unbounded_String then
         HTML.TR( " bgcolor=red " );
	    HTML.TD;
	       HTML.Print( "<FONT COLOR=YELLOW><BLINK>");
	       HTML.Break(" *** ERROR : " & To_String(Data.Error_Text) );
	       HTML.Print( "</BLINK></FONT>" );
	    HTML.TD_End;
	 HTML.TR_End;
	 Data.Error_Text := Null_Unbounded_String;
      end if;
   end Handle_Error_Text;

   --*****************************************************************************
   --|
   --|             A C D E    E N T R Y   P A N E L
   --|             ================================
   --|
   --| After login present the overview. The overview allows to select
   --| a project or to perform administration for projects or users
   --|
   --| Input Parameters
   --| ----------------
   --|    User_Name - Name of the user logged in
   --|
   --| Output Parameters
   --| -----------------
   --|                         
   --| cmd = CMD_Admin
   --|	  Maint_Action   = Maint_Project  - Project Maint. selected
   --|                     Project_Name   - Name of the project to be handled 
   --|	      "          = Maint_User     - User administration selected
   --|
   --| cmd = CMD_Project
   --|    Project_Name : name of the project selected
   --|
   --| cmd = *
   --|    User_Name      = name of logged in user.
   --|    Transaction_ID = Transaction identifier.
   --|
   function Show_Project( this : in Process.Object'Class ) return Boolean is
      dBase   : DB.Object := Data_Base( this );
      qID     : DB.Query_ID := Current_Query( this );

      pData   : Object_Data_Access  renames Admin.Object(this).Data;
      deb     : Debugging_Support.Handle := Debug( this );
      Project : Unbounded_String :=  DB.String_Field( dBase, qID, "name" );
   begin
      Enter( deb, Module & ".Show_Project");

      pData.Count := pData.Count + 1;

      Option( Text  => To_String(Project),
	      Value => HTML_String( To_String(Project) ) );

      Leave( deb, "");
      return True;
		     
   exception
      when Others =>
         Leave( deb, "Exception");
         return False;
   end Show_Project;

   --|
   --| This procedure builds the main entry panel for the 
   --| administration tasks.
   --|
   procedure Entry_Panel( this : in out Process.Object'Class ) is
      Data : Object_Data_Access renames Object(this).Data;      
   begin  
      HTML.Table( " width=100% " ) ; --- & HTML_String("#cccccc") );

         ------------------------------------------------------------------
         ---             PROJECTS AVAILABLE TO USER	                ---
         ------------------------------------------------------------------
         HTML.TR(" valign=top ");
            HTML.TD;
	       HTML.Heading(2,"Available Projects");
               Data.Count := 0;
               Selection(name => P_Project_Name, Size =>10, width=>70 );
               Query( this, 
	              SQL.SQL_Select( colum => D_Project_Name, 
		                      from  => T_Project_List),
		      Show_Project'Access );
               if Data.Count = 0 then
                  Option( Text => "*** no projects ***", Value => "0" );
               end if;
               Selection_End;

	       Break;
	    HTML.TD_End; 
         HTML.TR_End;
 
         -----------------------------------------------------------------
         ---                   MAINTENACE FUNCTIONS   	               ---
         -----------------------------------------------------------------
         HTML.TR( " valign=top " );
	    HTML.TD;
	       HTML.Heading(2,"Maintenance Functions");
               HTML.Table( " width=60% cols=2 " ); 
                  Radio("Select  Project",     V_Open_Project, " bgcolor=red ");
                  Radio("User Maintenance",    V_Maint_User,   " bgcolor=yellow ");
		  Radio("Role Administraton",  V_Maint_Role,   " bgcolor=yellow ");
	       	  Radio("System Parameters",   V_Maint_System, " bgcolor=yellow "); 
	          Radio("Project Maintenance", V_Maint_Project," bgcolor=yellow ");
	       HTML.Table_End;
	       Break;		   
	       Break;
	    HTML.TD_End;
         HTML.TR_End;

	 -----------------------------------------------------------------
	 ---                  HANDLE THE ERROR TEXT		       ---
	 -----------------------------------------------------------------
	 Handle_Error_Text( this );

      HTML.Table_End;

   end Entry_Panel;

   ---*****************************
   ---    PROJECT MAINTENANCE
   ---*****************************

   --|
   --|       A C D E    M A I N T E N A N C E   P A N E L
   --|       ============================================
   --|
   --|
   --| Output
   --|
   --| cmd = CMD_New_Project
   --|    		Owner_Name
   --|
   procedure Project_Maint_Panel( this : in out Process.Object'Class ) is
      data : Object_Data_Access := Instance( Admin.Object(This), ".User_Maint_Panel");
      Obj  : Project.Object renames Data.Project_Obj;
   begin
      Heading(2, "Project Data");

      HTML.Table;
         Field("Project Name", P_Project_Name, Name(Obj), 40, " bgcolor=red " ); 
         Field("Author",       P_Owner_Name,   Owner(Obj)  );
      HTML.Table_End;

      Heading(2, "Project Maintenance");
      HTML.Table( "bgcolor = red ");
         Radio("Create/Update Project", V_Update_Project );
	 Radio("Delete Project",        V_Delete_Project );
	 Radio("Show existing Project", V_Show_Project   );
      HTML.Table_End;

      Handle_Error_Text( this );

   end Project_Maint_Panel;

   --|	      
   --| Set the next panel to be displayed to project panel
   --|
   procedure Goto_Project_Maintenance( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Goto_Project_Maintenance");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Data.Next_Page  := Project_Maint_Panel'Access;
      Data.Next_Title := To_Unbounded_String("Project Maintenance");
   end Goto_Project_Maintenance;

   --|
   --| Handle the creation of new project or update an existing
   --| project.
   --|
   procedure Handle_Project_Update( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Handle_Project_Update");
      Debug : Debugging_Support.Handle renames Data.Debug;
      now   : Time := Clock;
   begin
      Enter( Debug, Module & ".Handle_Project_Update");

      if not Exists( Data.Project_Obj, Name(Data.Project_Obj) ) then
         Creation_Date( Data.Project_Obj, now );
      end if;
      Modification_Date( Data.Project_Obj, now );
 
      if not Exists( Data.User_Obj, Owner(Data.Project_Obj) )  then
         Data.Error_Text := To_Unbounded_String("Owner Name does not exist");
      else
         Update( Data.Project_Obj );
      end if;
 
      Goto_Project_Maintenance( this ); 
      Leave( Debug, "" );
   end Handle_Project_Update;

   --|
   --| Handle deletion of an existing project
   --|
   procedure Handle_Project_Delete( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Handle_Project_Delete");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Project_Delete");

      if not Exists( Data.Project_Obj, Name(Data.Project_Obj) )  then
         Data.Error_Text := To_Unbounded_String("Project does not exist");
      else
         Delete( Data.Project_Obj );
      end if;

 
      Goto_Project_Maintenance( this ); 
      Leave( Debug, "" );
   end Handle_Project_Delete;

   --|
   --| Show the data of an existing project
   --|
   procedure Handle_Project_Show( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Handle_Project_Show");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Project_Show");

      Load( Data.Project_Obj );
      if not Is_Valid( Data.Project_Obj ) then
         Data.Error_Text := To_Unbounded_String("Project not found");
      end if;

      Goto_Project_Maintenance( this ); 
      Leave( Debug, "" );
   end Handle_Project_Show;

   ---*****************************
   ---      USER MAINETNANCE
   ---*****************************

   --|			      
   --| Output
   --|
   --| cmd = CMD_User_Admin
   --|	     P_Next_Action = Create_User
   --|                     = Modify_User
   --|                     = Delete_User
   --|
   --| 
   procedure User_Maint_Panel( this : in out Process.Object'Class ) is
      data : Object_Data_Access := Instance( Admin.Object(This), ".User_Maint_Panel");
      User_Obj : User.Object renames Data.User_Obj;
   begin
      HTML.Table( " width=100% cols=1" );
      HTML.TR(" VALIGN=TOP ");

         HTML.TD;
            HTML.Table( " width=100% cols=2 bgcolor=" & HTML_String("#cccc99") );

               Field("Login Name", P_New_Login,       Login_Name( User_Obj),
	                                              aux => " bgcolor=red " );
               Field("First Name", P_Real_First_Name, Real_First_Name( User_Obj ) );
               Field("Last Name",  P_Real_Name,       Real_Last_Name( User_Obj ) );
               Field("Org. Unit",  P_Org_Unit,        Org_Unit( User_Obj ) );
               Field("Phone",      P_Phone,           Phone( User_Obj ) );
               Field("Location",   P_Location,        Location( User_Obj ) );
               Field("e-mail",     P_Email,           Email( User_Obj ) );

            HTML.Table_End;
         HTML.TD_End;

         HTML.TD;
            HTML.Table( " width=100% cols=2 bgcolor=red " );	          
               Radio( "New/Update User ", V_Create_User);
               Radio( "Get User Information", V_Show_user );
               Radio( "Delete User ", V_Delete_User );
            HTML.Table_End;
         HTML.TD_End;
      HTML.TR_End;

      Handle_Error_Text( this );

      HTML.Table_End;
		     
   end User_Maint_Panel;
     	    

   --|
   --| This is the project maintenance command handler
   --|
   procedure Goto_User_Maintenance( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Handle_User_Maintenace");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Data.Next_Page := User_Maint_Panel'Access;
      Data.Next_Title := To_Unbounded_String("User Maintenace");
   end Goto_User_Maintenance;
					  
   --|
   --| Create a new user or update a known user
   --|
   procedure Handle_Create_User( this : in out Object) is
      Data     : Object_Data_Access := Instance( This, ".Handle_Create_User");
      Debug    : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Create_User");

      Update( Data.User_Obj );

      Goto_User_Maintenance( this );
      Leave( Debug, "" );
   end Handle_Create_User;

   --|		      	       
   --| Create a new user or update a known user
   --|
   procedure Handle_Delete_User( this : in out Object) is
      Data     : Object_Data_Access := Instance( This, ".Handle_Create_User");
      Debug    : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Delete_User");

      Delete( Data.User_Obj );

      User.Finalize( Data.User_Obj );
      User.Initialize( Data.User_Obj, Data.Debug );

      Goto_User_Maintenance( this );
      Leave(Debug, "" );
   end Handle_Delete_User;

   --|
   --| Modify the user data
   --|
   procedure Handle_Show_User( this : in out Object) is
      Data     : Object_Data_Access := Instance( This, ".Handle_Create_User");
      Debug    : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Delete_User");
 
      Load( Data.User_Obj );
      if not Is_Valid( Data.User_Obj ) then
         Data.Error_Text := To_Unbounded_String("User not found");
      end if;

      Goto_User_Maintenance( this );
      Leave(Debug, "" );
   end Handle_Show_User;

   ---*****************************
   ---      PROJECT 
   ---*****************************

   --|
   --| Open a the currently selected project
   --|
   procedure Handle_Open_Project( this : in out Object) is
      Data  : Object_Data_Access := Instance( This, ".Handle_Open_Project");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Handle_Open_Project");
      Leave( Debug, "" );
   end Handle_Open_Project;



   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Preparation code as required by the base process interface.
   --|
   procedure Prepare( this  : in out Object ) is
      Data : Object_Data_Access;
   begin
      Data := new Object_Data;
      this.Data := Data;

      Data.Debug := Debug(this);
 
      Component( This, P_Next_Action,       Optional );
      Component( This, P_Project_Name,      Optional );
      Component( This, P_New_Login,         Optional );
      Component( This, P_Owner_Name,        Optional );
      Component( This, P_Org_Unit,          Optional );
      Component( This, P_Phone,             Optional );
      Component( This, P_Login_Name,        Optional );
      Component( This, P_Real_Name,         Optional );
      Component( This, P_Real_First_Name,   Optional );
      Component( This, P_User_Action,       Optional );
      Component( This, P_Location,          Optional );
      Component( This, P_Email,		    Optional );

      Data.Next_Page  := Entry_Panel'Access;
      Data.Next_Title := To_Unbounded_String("Main Panel");

      -- Initialize and Assign the user object to a database 
      User.Initialize( Data.User_Obj, Data.Debug );
      Data_Base( Data.User_Obj, Data_Base(this) );

      -- Initialize project objects
      Project.Initialize( Data.Project_Obj, Data.Debug );
      Data_Base( Data.Project_Obj, Data_Base(this) );

   end Prepare;

   --|
   --| Translate user actions
   --|
   procedure Perform( this : in out Object ) is
      Data        : Object_Data_Access  := Instance( this, ".Perform" );
      Debug       : Debugging_Support.Handle renames Data.Debug;
      Action      : Unbounded_String := Component( this, P_Next_Action );
      Cmd_Handler : Action_Handler := null;

      cmds : Commands := (
        --- Open a project for work
      	( To_Unbounded_String(V_Open_Project), 	Handle_Open_Project'Access),

	--- Handle project related commands
	( To_Unbounded_String(V_Maint_Project), Goto_Project_Maintenance'Access ),
	( To_Unbounded_String(V_Update_Project),Handle_Project_Update'Access ),
	( To_Unbounded_String(V_Delete_Project),Handle_Project_Delete'Access ),
	( To_Unbounded_String(V_Show_Project),  Handle_Project_Show'Access ),

	--- Handle user maintenance related commands
	( To_Unbounded_String(V_Maint_User),	Goto_User_Maintenance'Access	),
	( To_Unbounded_String(V_Create_User),	Handle_Create_User'Access ),
	( To_Unbounded_String(V_Show_User),	Handle_Show_User'Access ),
	( To_Unbounded_String(V_Delete_User),	Handle_Delete_User'Access )
	);

   begin
      Enter( Debug, Module & ".Perform");

      -- Load the user attributes
      Login_Name     ( Data.User_Obj, To_String(Component( This, P_New_Login )) );
      Real_Last_Name ( Data.User_Obj, To_String(Component( This, P_Real_Name )) );
      Real_First_Name( Data.User_Obj, To_String(Component( This, P_Real_First_Name)));
      Login_Name     ( Data.User_Obj, To_String(Component( This, P_New_Login )) );
      Phone          ( Data.User_Obj, To_String(Component( This, P_Phone     )) );
      Location       ( Data.User_Obj, To_String(Component( This, P_Location  )) );
      Email          ( Data.User_Obj, To_String(Component( This, P_Email     )) );
      Org_Unit       ( Data.User_Obj, To_String(Component( This, P_Org_Unit  )) );

      -- Load the project attributes
      Name      ( Data.Project_Obj, Component( This, P_Project_Name ) );
      Owner     ( Data.Project_Obj, Component( This, P_Owner_Name   ) );
	     
      -- Try to identify the command handler
      for i in 1..Cmds'Length loop
         if Cmds(i).Name = Action then
	    Cmd_Handler := Cmds(i).Handler;
	    exit;
	 end if;
      end loop;

      if Cmd_Handler /= null then
         Cmd_Handler( this );
      else
      	 CGI.Put_Variables;
      end if;
 
      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Perform" );
         raise;
   end Perform;
			  
   --|
   --| Display the currently selected panel as required by	    
   --| the base process interface.
   --|
   procedure Display( this : in out Object ) is
      Data : Object_Data_Access        := Instance( this, ".Display" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Display");

      if Data.Next_Page /= null then
         Page( this, Data.Next_Page, To_String(Data.Next_Title) );
      end if;

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Display" );
         raise;
   end Display;

   --|
   --| Handle a cancel command. Required by the base process
   --| interface.
   --|
   procedure Cancel( this : in out Object ) is
      Data : Object_Data_Access        := Instance( this, ".Cancel" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Cancel");

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Cancel" );
         raise;
   end Cancel;

end DLG.Admin;

