--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/docgen.adb,v $
--| Description     : generate a documentation page from ada specification
--| Author          : Michael Erdmann
--| Created On      : 29.12.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.1 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--|
--| This process calls an ada parser an generates an HTML output 
--| document.      
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--| [1]  CGI - Baseprocess documentation
--|
--| History
--| =======
--| $Log: docgen.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|

---- Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Text_IO;                  use Ada.Text_IO;
with Ada.Strings.Unbounded;        use Ada.Strings.Unbounded;
with Unchecked_Deallocation;

with CGI;

---- ASCL Components
with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

---- STD Process
with Environement;                 use Environement;
with STD.Process;  		   use STD.Process;                     
with STD.CGI_Debug;

use  STD;


---- Common Components
with List;			   use List;
with Interface.List_Element;	   use Interface.List_Element;	   
use  Interface;

with HTML;			   use HTML;  

---- Local components
with Parser;			   use Parser;
with Contents_Element;		   use Contents_Element;
with Text_Block;		   use Text_Block;


package body Docgen is

   Version : constant String := "$Id: docgen.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "docgen.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Transaction : Positive                 := 1;
	 Count       : Integer                  := 0;
	 Debug       : Debugging_Support.Handle := null;
	 Name        : Unbounded_String ;
	 Next_Page   : Customized_Page_Handler   := null;
	 Next_Title  : Unbounded_String;

	 Pkg_Info    : List.Object;
      end record;
   
   ---=====================================================================---
   ---===              L O C A L    D A T A    T Y P E S	        ===---
   ---=====================================================================---

   --- External component names
   P_Next_Action    : constant String := "NextAction";
   P_File_Name      : constant String := "FileName";
    
   --- Constants which are derived from the input forms
 
   V_Classes	    : constant String := "Classes";        -- Next_Action
   V_Overview	    : constant String := "Overview";       -- Next_Action

   type Action_Handler is access procedure ( this : in out Object );

   type Action_Record is record
         name    : Unbounded_String := Null_Unbounded_String;
         handler : Action_Handler   := null;
      end record;

   type Commands is array( Positive range <> ) of Action_Record;


   type Key_Words is array ( Positive range <> ) of Unbounded_String;

   Package_Keywords : constant Key_Words := (
   			To_Unbounded_String("Functional Description"),
			To_Unbounded_String("History"),
			To_Unbounded_String("Component Data"),
			To_Unbounded_String("Oparations on")
			);

   Method_keywords : constant Key_Words := (
   			To_Unbounded_String("Description"),
			To_Unbounded_String("Preconditions"),
			To_Unbounded_String("Postconditions"),
			To_Unbounded_String("Exceptions"),
			To_Unbounded_String("Note")
			);

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Retrieve the object instance data
   --|
   function Instance( this : Object; info : String ) return Object_Data_Access is
   begin
      if this.data = null then
         raise Not_Initialized;	       
      end if;
      return this.Data;
   exception
      when The_Error : Others =>
         Error( this, The_Error, Module & ".Instance for " & info );
	 raise;
   end Instance;

   ---======================================================================---

   --|
   --| Locate a given file name. The function return the full path
   --| of the file.
   --|
   function Locate( name : Unbounded_String ) return Unbounded_String is
   begin
      return To_Unbounded_String("/home/ACDE/tt/") & name;
   end Locate;

   --|
   --| Make a tag symbol
   --|
   function Make_Tag( nbr : Integer ) return String is
   begin
      return "Tag" & Integer'Image(nbr);
   end Make_Tag;

   --|
   --| Format a comment block by checking for the listed key words
   --|
   procedure Format( text : Text_Array_Access; keys : Key_Words ) is
      function Is_Key( text : Unbounded_String ) return Boolean is
      begin
         for i in 1..keys'Length loop
	    if Index( text, To_String(keys(i))) /= 0 then
	       return True;
	    end if;
	 end loop;
	 
	 return False; 
      end Is_Key;

   begin
      for i in 1..text.all'Length loop
         if Is_Key( text(i) ) then
	    Heading( 3, To_String(text(i)) );
	 else
	    if Index( text(i), " =====") = 0 then
	        Break( To_String(text(i)) );
	    end if;
	 end if;
      end loop;
   end ;

   ---======================================================================---
   ---===                H T M L      P A N E L S			 ===---
   ---======================================================================---

   --  ****************************
   --      Main Entry Panel
   --  ****************************
   procedure Get_Package_Name( this : in out Process.Object'Class ) is
   begin
      Put_Line("Filename:" ); 
      Text(Name => P_File_Name, Value => "", Size => 40 );

      Break;
      Put_Line( "Overview   ");
      Radio_Button( Name => P_Next_Action, Value => V_Overview ); Break;
      Put_Line( "Classes    ");
      Radio_Button( Name => P_Next_Action, Value => V_Classes ); Break;

   end Get_Package_Name;


   --  ****************************
   --    PACKAGE INFORMATION PAGE
   --  ****************************

   --|
   --| Format the object for output.
   --|
   procedure Print_Component( Data : in Contents )  is
   begin
      case Data.What is
      	 when Package_Unit => 
            Heading(2, "Package " & To_String( Data.Name ) );
            if Data.Comments /= null then
	       Format( Data.Comments, Package_Keywords );
            end if;
	    
            if Data.Libraries /= null then
	       Heading( 3, "Used Components");	     
               for i in 1..Data.Libraries.all'Length loop
	          Break( To_String(Data.Libraries(i)) );
	       end loop;
            end if;

	 when Method => 
	    if Data.Is_Abstract then
	       Heading(2, "Abstract Method : " & To_String(Data.Name) );
	    else
               Heading(2, "Method : " & To_String( Data.Name ) );
	    end if;

            if Data.Comments /= null then
	       Format( Data.Comments, Method_Keywords );
            end if;

	    if Data.Arguments /= Null_Unbounded_String then
	       Heading( 3, "Arguments");
	       Break( To_String(Data.Arguments) );
	       Break;
	    end if;

	    if Data.Returns /= Null_Unbounded_String then
	       Heading( 3, "Returns");
	       Break( To_String(Data.Returns) );
	       Break;
	    end if;

	 when Class_or_Type =>
	    null;
      end case;

   end Print_Component;


   --|
   --| Build the package description and an index
   --| entries.
   --|
   procedure Display_Index( Info : in out List.Object ) is
      Data      : Contents;
      item      : List_Element.Handle := null;
      Nbr       : Integer := 1;
   begin 
      -- print out the package header
      item := Get( Info, First );
      while item /= null loop
         Data := Contents_Element.Value(item);
	  
	 if Data.What = Package_Unit then
            Print_Component( Data );
	    exit;
	 end if;
         item := Get( Info );
	 Nbr := Nbr + 1;
      end loop;
  
      -- build the index for all other components.
      Heading(3, "Methods and Types");
      item := Get( Info, First );
      while item /= null loop
         Data := Contents_Element.Value( item );
	  
	 if Data.What /= Package_Unit then
	    Link( "#" & Make_Tag(Nbr), To_String( Data.Name ) ); Break;
	 end if;

         item := Get( Info );
	 Nbr := Nbr + 1;
      end loop;

   end Display_Index;

   --|
   --| Display the contents information stored in the 
   --| given list.
   --|
   procedure Display_Contents( Info : in out List.Object ) is
      Data      : Contents;
      item      : List_Element.Handle := null;
      Nbr       : Integer := 1;
   begin 
      Display_Index( Info );

      item := Get( Info, First );
      while item /= null loop
         Data := Contents_Element.Value( item ); 
	    
	 if Data.What /= Package_Unit then 
	    Tag( Make_Tag(Nbr) );
            Print_Component( Data );
	 end if;
	 Line;

         item := Get( Info );
	 Nbr := Nbr + 1;
      end loop;

   end Display_Contents;

   --|
   --| Build the overview panel
   --|
   procedure Display_Package( this : in out Process.Object'Class ) is
      Data : Object_Data_Access := Instance( Object(this), ".Display_Package");
   begin
      Display_Contents( Data.Pkg_Info );
      Finalize( Data.Pkg_Info );
   end Display_Package;

   ---======================================================================---
   ---===                 C O M M A N D    H A N D L E R S		 ===---
   ---======================================================================---

   --|
   --| Generate a component overview
   --|
   procedure Generate_Overview( this : in out Object ) is
      Data : Object_Data_Access := Instance( this, ".Generate_Overview");

      File_Name : Unbounded_String := Component(this, P_File_Name);
      Path      : Unbounded_String;
      P		: Parser.Object; 
   begin
      Initialize( P, Data.Debug );

      Path := Locate( File_Name );
      Parse( P, To_String( Path ) );
      Data.Pkg_Info := Result(P);
      
      Data.Next_Page  := Display_Package'Access;
      Data.Next_Title := To_Unbounded_String("Package ") & File_Name;

      Finalize( P );
   end Generate_Overview;          

   --|
   --| Generate a classes overview
   --|
   procedure Generate_Classes( this : in out Object ) is
   begin
      Put_Line( "Classes");
   end Generate_Classes;
  
   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Prepare for execution by instaling the component names
   --|
   procedure Prepare( this  : in out Object ) is
      Data : Object_Data_Access;
   begin
      Data := new Object_Data;
      this.Data := Data;

      Data.Debug := Debug(this);
 
      Component( This, P_Next_Action,       Optional );
      Component( This, P_File_Name,         Optional );

      Data.Next_Page  := Get_Package_Name'Access;
      Data.Next_Title := To_Unbounded_String("Entry Panel");
   end Prepare;

   --|
   --| Perform a command
   --|
   procedure Perform( this : in out Object ) is
      Data        : Object_Data_Access  := Instance( this, ".Perform" );
      Debug       : Debugging_Support.Handle renames Data.Debug;
      Action      : Unbounded_String := Component( this, P_Next_Action );
      Cmd_Handler : Action_Handler := null;

      cmds : Commands := (
      	( To_Unbounded_String(V_Overview), Generate_Overview'Access ),
	( To_Unbounded_String(V_Classes),  Generate_Classes'Access )
	);

   begin
      Enter( Debug, Module & ".Perform");

      for i in 1..Cmds'Length loop
         if Cmds(i).Name = Action then
	    Cmd_Handler := Cmds(i).Handler;
	    exit;
	 end if;
      end loop;

      if Cmd_Handler /= null then
         Cmd_Handler( this );
      else
      	 CGI.Put_Variables;
      end if;
 
      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Perform" );
         raise;
   end Perform;

   --|
   --| Display output
   --|
   procedure Display( this : in out Object ) is
      Data  : Object_Data_Access        := Instance( this, ".Display" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Display");

      if Data.Next_Page /= null then
         Page( this, Data.Next_Page, To_String(Data.Next_Title) );
      end if;

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Display" );
         raise;
   end Display;

   --|
   --| Cancel an action
   --|
   procedure Cancel( this : in out Object ) is
      Data : Object_Data_Access        := Instance( this, ".Cancel" );
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Cancel");

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Cancel" );
         raise;
   end Cancel;

end Docgen;

