--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/list.adb,v $
--| Description     : User Object
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--| Functional Description
--| ======================
--| This is the implementation of a simple list. It relies on
--| the features as specified in the Interface.List_Element
--| package.
--|
--| Restrictions
--| ============   
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: list.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Unchecked_Deallocation;

with Interface.List_Element;       use Interface.List_Element;
use  Interface;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
use  ASCL;

package body List is

   Version : constant String := "$Id: list.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "list.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Debug   : Debugging_Support.Handle := null;
	 Last    : List_Element.Handle := null;
	 First   : List_Element.Handle := null;
	 Read_Ptr: List_Element.Handle := null;
      end record;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object'Class; info : String ) 
                                               return  Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;

   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Intialize the Object data and call the Initalization
   --| procedure of the extention.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug; 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extention.
   --|
   procedure Finalize( this : in out Object'Class ) is
      Data   : Object_Data_Access       := This.Data;
      Debug  : Debugging_Support.Handle := Data.Debug;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      Enter( Debug, Module & ".Finalize");

      if Data /= null then
         Free( Data );

         This.Data := null;
      end if;

      Leave(Debug,"");

   exception
      when The_Error : Others =>
         Leave(Debug, "**exception**");
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Get the handle of the first element
   --|
   function First( this : in Object'Class ) return List_Element.Handle is
      Data : Object_Data_Access := Instance( this, ".Get");
   begin
      return Data.First;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".First");
         raise;
   end First;

   --|
   --| get the handle to the last element.
   --|
   function Last( this : in Object'Class ) return List_Element.Handle is
      Data : Object_Data_Access := Instance( this, ".Get");
   begin
      return Data.Last;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".First");
         raise;
   end Last;


   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Purpose:
   --|    Append the given elements to the list.
   --|
   --| Pre-Conditions:
   --|    List has been initialized
   --|
   --| Post-Conditions:
   --|	  none
   --|
   procedure Append( this : in out Object'Class; 
                     item : in List_Element.Object'Class) 
   is
      Data  : Object_Data_Access := Instance( this, ".Append" );   
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & ".Append");

      if Data.First = null then
         Data.First := Reference( Item );
	 Data.Last  := Data.First;
      else
         Next( Data.Last.all, Reference( item ) );
	 Data.Last := Reference( item );
      end if;
   
      Leave( Debug, "");

   exception
      when The_Error : Others =>
         Leave( Debug, "*** exception ***");
         Error( this, The_Error, ".Append" );
         raise;
   end Append;

   --|
   --| Purpose:
   --|   Perform a ceratin action on all elements within
   --|   the list.
   --|
   --| Pre-Conditions:
   --|   List is initialized.
   --|
   --| Post-Conditions:
   --|   none.
   --|
   procedure Perform( this : in out Object'Class; 
                      cntx : in out Iterator_Type ) 
   is
      Data    : Object_Data_Access  := Instance( this, ".Append" );
      Debug   : Debugging_Support.Handle renames Data.Debug;
      current : List_Element.Handle := Data.First;   
   begin
      Enter( Debug, Module & ".Perform");

      while current /= null loop
         Process( cntx, current.all );

         current := Next( current.all );
      end loop;

      Leave( Debug, "");

   exception
      when The_Error : Others =>
         Leave( Debug, "*** Exception ***");
         Error( this, The_Error, ".Perform" );
         raise;
   end Perform;
 
   --|
   --|
   --|
   function Get( this : in Object'Class; mode : Get_Mode := Next )
                                        return List_Element.Handle 
   is
      Data    : Object_Data_Access  := Instance( this, ".Append" );
      Debug   : Debugging_Support.Handle renames Data.Debug;
   begin
      if mode = First then
         Data.Read_Ptr := Data.First;
      else
         if Data.Read_Ptr /= null then
            Data.Read_Ptr := Next( Data.Read_Ptr.all );
	 end if;
      end if;

      return Data.Read_Ptr;
   end Get;

end List;








