--                              -*- Mode: Ada -*-
-- Filename        : $Id: parser.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $
-- Description     : ada parser
-- Author          : Michael Erdmann
-- Created On      : Mon Apr 13 21:32:28 1998
-- Last Modified By: $Author: ACDE $
-- Last Modified On: $Date: 2000/01/21 17:18:23 $
-- Update Count    : 0
-- Status          : $State: Released $
--
-- Functional Description
-- ======================
-- This is a simple ada parser. 
--
-- History
-- =======
-- $Log: parser.adb,v $
-- Revision 1.1  2000/01/21 17:18:23  ACDE
-- Release Version test_1_0
--
-- Revision 1.2  2000/01/16 16:06:17  ACDE
-- Release Version V8_1test
--
-- Revision 1.1  1999/12/30 09:11:09  ACDE
-- first tests for docgen completed
-- 
--
--
with Ada.Text_IO;		   use Ada.Text_IO;
with Ada.Characters.Latin_1;       use Ada.Characters;
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Strings;                  use Ada.Strings;
with Ada.Strings.Fixed;            use Ada.Strings.Fixed;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;

with Unchecked_Deallocation; 

with ASCL;
with ASCL.Debugging_Support;	   use ASCL.Debugging_Support;
use  ASCL;


with Interface.List_Element;       use Interface.List_Element;
use  Interface;

with List;			   use List;

with Contents_Element;             use Contents_Element;
with Text_Block;		   use Text_Block;
with HTML;			   use HTML;

package body Parser is

   package TI renames Ada.Text_IO;

   subtype String_Range_Type is Integer range 1..48;
   subtype Simple_Identifier_Type is String( String_Range_Type );

   Blank_Identifier : constant Simple_Identifier_Type := ( others => ' ');

   type Lexical_Unit_Type is -- the set of valid Ada Lexical tokens
      ( Identifier_Lex, Numeric_Literal_Lex,   Character_Literal_Lex,  String_Lex,
        Delimiter_Lex,  Comment_Lex,           Unknown_Lex,            End_Of_File_Lex );


   type Token_Type is -- a lexical token type and its character representation
      record
         Lexicon      : Simple_Identifier_Type;
         Lexical_Unit : Lexical_Unit_Type := Unknown_Lex;
      end record;

   type Ada_Symbol_Type is

      ( Abort_Symbol,    Abs_Symbol,        Accept_Symbol,    Access_Symbol,    All_Symbol,
        And_Symbol,      Array_Symbol,      At_Symbol,        Begin_Symbol,     Body_Symbol,
        Case_Symbol,     Constant_Symbol,   Declare_Symbol,   Delay_Symbol,     Digits_Symbol,
        Do_Symbol,
        Else_Symbol,     Elsif_Symbol,      End_Symbol,       Entry_Symbol,     Exception_Symbol,
        Exit_Symbol,     For_Symbol,        Function_Symbol,  Generic_Symbol,   Goto_Symbol,
        If_Symbol,       In_Symbol,         Is_Symbol,        Limited_Symbol,   Loop_Symbol,
        Mod_Symbol,      New_Symbol,        Not_Symbol,       Null_Symbol,      Of_Symbol,
        Or_Symbol,       Others_Symbol,     Out_Symbol,       Package_Symbol,   Pragma_Symbol,
        Private_Symbol,  Procedure_Symbol,  Raise_Symbol,     Range_Symbol,     Record_Symbol,
        Rem_Symbol,      Renames_Symbol,    Return_Symbol,    Reverse_Symbol,   Select_Symbol,
        Separate_Symbol, Subtype_Symbol,    Task_Symbol,      Terminate_Symbol, Then_Symbol,
        Type_Symbol,     Use_Symbol,        When_Symbol,      While_Symbol,     With_Symbol,
        Xor_Symbol,      Abstract_Symbol,
        
        Identifier_Symbol, 
        Character_Symbol, 
        Comment_Symbol,   
        Numeric_Symbol,
        String_Symbol,
        
        Equal_Symbol,    Period_Symbol,     Asterix_Symbol,   Colon_Symbol,     Slash_Symbol, 
        Greater_Symbol,  Less_Symbol,       Ampersand_Symbol, LPara_Symbol,     RPara_Symbol,
        Plus_Symbol,     Comma_Symbol,      Minus_Symbol,     Semicolon_Symbol, Bar_Symbol,
        
        Eq_Gr_Symbol,    Period_2_Symbol,   Asterix_2_Symbol, Assign_Symbol,    NotEqual_Symbol,
        Gr_Eq_Symbol,    Gr_Gr_Symbol,      Less_Eq_Symbol,   Less_2_Symbol,    Less_Gr_Symbol,
        
        Tick_Symbol,     Unknown_Symbol );

   type Ada_Symbol_Set_Type is
      array ( Ada_Symbol_Type'First .. Ada_Symbol_Type'Last ) of Boolean;

   Syntax_Error : exception;


   Accept_Statement_Terminators  : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( Do_Symbol | 
                              Semicolon_Symbol | 
                              Unknown_Symbol => True,
                              others            => False );

   Body_Starters : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( Accept_Symbol |
                              Begin_Symbol |
                              Case_Symbol | 
                              End_Symbol | 
                              Function_Symbol | 
                              Generic_Symbol | 
                              Identifier_Symbol | 
                              If_Symbol | 
                              Loop_Symbol | 
                              Package_Symbol | 
                              Procedure_Symbol | 
                              Raise_Symbol |
                              Record_Symbol | 
                              Select_Symbol |
                              Task_Symbol | 
                              Unknown_Symbol => True,
                              others            => False );

   Formal_Parameter_Terminators : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( LPara_Symbol | 
                              RPara_Symbol | 
                              Unknown_Symbol => True,
                              others            => False );

   Generic_Starters : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( Function_Symbol | 
                              Package_Symbol | 
                              Procedure_Symbol | 
                              With_Symbol | 
                              Unknown_Symbol => True,
                              others            => False );

   Package_Spec_Starters : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( Case_Symbol | 			     --'
                              End_Symbol | 
                              Function_Symbol | 
                              Generic_Symbol | 
                              Package_Symbol | 
                              Procedure_Symbol | 
                              Record_Symbol | 
                              Task_Symbol | 
                              Unknown_Symbol => True,
                              others            => False );

   Task_Spec_Starters : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( End_Symbol        | 		     --'
                              Entry_Symbol | 
                              Unknown_Symbol   => True,
                              others              => False );

   Type_Mark_Terminators : constant Ada_Symbol_Set_Type 
     := Ada_Symbol_Set_Type'( Is_Symbol        | 		    --'
                              Semicolon_Symbol | 
                              Unknown_Symbol   => True,
                              others              => False );

   subtype Delimiter_Range_Type is    
     Ada_Symbol_Type range Equal_Symbol .. Unknown_Symbol;
   
   type Delimiter_Table_Type is
      array ( Delimiter_Range_Type ) of Simple_Identifier_Type;

   subtype Reserved_Word_Range_Type is 
     Ada_Symbol_Type range Abort_Symbol .. Identifier_Symbol;
   
   type Reserved_Word_Table_Type is
      array ( Reserved_Word_Range_Type ) of Simple_Identifier_Type;
   
   Delimiter_Table : Delimiter_Table_Type :=
     ("=                                               ",".                                               ",
      "*                                               ",":                                               ",
      "/                                               ",">                                               ",
      "<                                               ","&                                               ",
      "(                                               ",")                                               ",
      "+                                               ",",                                               ",
      "-                                               ",";                                               ",
      "|                                               ","=>                                              ",
      "..                                              ","**                                              ",
      ":=                                              ","/=                                              ",
      ">=                                              ",">>                                              ",
      "<=                                              ","<<                                              ",
      "<>                                              ","'                                               ",
      "                                                ");

   Reserved_Word_Table : Reserved_Word_Table_Type
     := 
     ("abort                                           ","abs                                             ",
      "accept                                          ","access                                          ",
      "all                                             ","and                                             ",
      "array                                           ","at                                              ",
      "begin                                           ","body                                            ",
      "case                                            ","constant                                        ",
      "declare                                         ",
      "delay                                           ","digits                                          ",
      "do                                              ","else                                            ",
      "elsif                                           ","end                                             ",
      "entry                                           ","exception                                       ",
      "exit                                            ","for                                             ",
      "function                                        ","generic                                         ",
      "goto                                            ","if                                              ",
      "in                                              ","is                                              ",
      "limited                                         ","loop                                            ",
      "mod                                             ","new                                             ",
      "not                                             ","null                                            ",
      "of                                              ","or                                              ",
      "others                                          ","out                                             ",
      "package                                         ","pragma                                          ",
      "private                                         ","procedure                                       ",
      "raise                                           ","range                                           ",
      "record                                          ","rem                                             ",
      "renames                                         ","return                                          ",
      "reverse                                         ","select                                          ",
      "separate                                        ","subtype                                         ",
      "task                                            ","terminate                                       ",
      "then                                            ","type                                            ",
     "use                                             ","when                                            ",
     "while                                           ","with                                            ",
     "xor                                             ","abstract                                        ",
     "                                                " );

   Space : constant Character := ' ';
   Tab   : constant Character := ' ';

   subtype String_Line_Range_Type is -- a single line of source code
      Integer range 1 .. 132;


   ---=====================================================================---
   ---=== OBJECT DATA =====================================================---
   ---=====================================================================---

   type Object_Data is record
      Current_Line_Count      : Integer := 0;
      Current_Symbol          : Ada_Symbol_Type;
      Current_Token           : Token_Type;
      Next_Token              : Token_Type;
      Next_Ada_Symbol         : Ada_Symbol_Type;
      First_Pragma_Identifier : Boolean := True;

      Last_Character_In_Line  : Integer := 0;
      Current_Character       : Character := Space;
      Input_Line : String( String_Line_Range_Type ) := ( others => Space );
      -- keeps track of the current character position in Input_Line
      Line_Pointer : String_Line_Range_Type := String_Line_Range_Type'Last ;

      -- is a simple allready opened flag: prevents open errors
      File_Is_Open            : Boolean  := False;
      Input_File              : TI.File_Type;
      Output_File             : TI.File_Type;

      Name                    : String(1..1024);

      Debug		      : Debugging_Support.Handle := null;
      Comments                : Text_Block.Object;
      Args		      : Unbounded_String := Null_Unbounded_String;
      Return_Type             : Unbounded_String := Null_Unbounded_String;

      Libraries		      : Text_Block.Object;
      Result                  : List.Object;
   end record;


   ---======================================================================---
   ---==== PRIVATE PROCEDURES ==============================================---
   ---======================================================================---

   function Initialize return Object_Data_Access is 
   begin
      return new Object_Data;
   end Initialize;

   procedure Free is 
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);


   procedure Enter_Scope( O : in out Object'Class; name : String ) is
   begin
      null;
   end Enter_Scope;

   procedure Exit_Scope( O : in out Object'Class ) is
   begin
      null;
   end Exit_Scope;

   ---------------------------------------------------------------------------
   procedure NextCh( This : in Object_Data_access ) is
   --|Description>
   --|  Identifies the next available character in the input stream. Performs
   --|  following: end-of-line => space, funny characters => space, 
   --|  end-of-file => EOT, 'A'..'Z' => 'a'..'z'.
   --||Description>
   begin
   
      -- Time to read new line of text

      if This.Line_Pointer > This.Last_Character_In_Line then
         declare -- yes
            Chars : Integer range 0 .. String_Line_Range_Type'last;
         begin
            TI.Get_Line( This.Input_File, This.Input_Line, Chars );
               -- Bump the metrics stuff
            This.Current_Line_Count := This.Current_Line_Count + 1;

            -- Initialize stuff after new line

            This.Current_Character      := Space;
            This.Last_Character_In_Line := Chars;
            This.Line_Pointer           := 1;

         -- Resolve problems

         exception

            when TI.Data_Error =>
               Trace( This.Debug, "<<<LA.NextCH>>> Data Exception");
               This.Current_Character := ASCII.EOT;

            when TI.End_Error  =>
                This.Current_Character := ASCII.EOT;

            when TI.Mode_Error =>
               Trace( This.Debug, "<<<LA.NextCH>>> Mode Exception");
               This.Current_Character := ASCII.EOT;

            when others     =>
               Trace( This.Debug, "<<<LA.NextCH>>> Others Exception");
               This.Current_Character := ASCII.EOT;
         end;

      else
         -- Continue to parse the current good line of source
         This.Current_Character := This.Input_Line(This.Line_Pointer);
         This.Line_Pointer      := This.Line_Pointer + 1;

         -- Convert funnies to spaces
         if This.Current_Character <= ASCII.US then
            This.Current_Character := Space;

         -- Convert upper to lower
         elsif This.Current_Character in 'A' .. 'Z' then
            This.Current_Character :=  Character'Val(Character'Pos('a') +
                                  Character'Pos(This.Current_Character) -
                                  Character'Pos('A'));
         end if;
      end if;

   end NextCh;


-- GET TOKEN ---------------------------------------------------------------

   procedure Get_Token( This : in Object_Data_access; Token : out Token_Type ) is
   --|Description>
   --|  Procedure which gets the next token (a record
   --|  of its lexical unit type and its integer
   --|  string value).
   --||Description>

      CH     : String_Line_Range_Type := 1;

      -- SAVE CHARACTER ----------------------------------------------------

      procedure Save_Character is
      --|Description>
      --|  Procedure which saves the current character,
      --|  indexed by its lexicon value (integer string
      --|  value), and calls the Nextch procedure to get
      --|  the next character.
      --||Description>
      begin
         -- Add character to string reprsentation if within limits
         if CH < String_Range_Type'last then

            Token.Lexicon(CH) := This.Current_Character;
            CH                := CH + String_Line_Range_Type(1);

         end if;
         NextCh(This);

      end Save_Character;
			 

      -- GET IDENTIFIER ------------------------------------------------------
                              
      procedure Get_Identifier is
      --|Description>
      --|  Valid Ada identifiers are constructed from the
      --|  character stream.
      --||Description>
      begin
         Token.Lexical_Unit := Identifier_Lex;
         Token.Lexicon      := Blank_Identifier;
         Save_Character;

         -- Add characters to the representation until invalid char found
         loop
            if (This.Current_Character not in 'a'..'z') and
               (This.Current_Character not in '0'..'9') and 
               (This.Current_Character /= '_') then
               exit;
            else
               Save_Character;
            end if;
         end loop;

      end Get_Identifier;


      -- GET NUMERIC LITERAL ------------------------------------------

      procedure Get_Numeric_Literal is
      --|Description>
      --|  Procedure which searches for integer, exponential,
      --|  and hexadecimal values and then calls the 
      --|  appropriate procedures to save them as their
      --|  lexicon values (integer string values).
      --||Description>

         -- GET INTEGER ------------------------------------------------
         procedure Get_Integer is
         --|Description>
         --|  Procedure which checks to see if the
         --|  current character is an integer; if so,
         --|  if calls the Save_Character procedure
         --|  to save the character as its lexicon
         --|  value (integer string value).
         --||Description>
         begin

            loop
               if   (This.Current_Character in '0'..'9') 
                 or (This.Current_Character = '_') then
                  Save_Character;
               else
                  exit;
               end if;
            end loop;

         end Get_Integer;
    
         -- GET EXPONENT -------------------------------------------------
         procedure Get_Exponent is
         --|Description>
         --|  Procedure which identifies exponential
         --|  characters and calls the Save_Character
         --|  procedure to save them as their lexicon
         --|  value (integer string value).
         --||Description>
         begin
            Save_Character;
            if  (This.Current_Character = '+') or 
                (This.Current_Character = '-') 
            then
               Save_Character;
            end if;
            Get_Integer;
         end Get_Exponent;


         -- GET EXTENDED INTEGER -----------------------------------
         procedure Get_Extended_Integer is
         --|Description>
         --|  Procedure which searches for hexadecimal values
         --|  and calls the Save_Character procedure to save
         --|  them as their lexicon values (integer string
         --|  values).
         --||Description>
         begin
            loop
              if (This.Current_Character in '0'..'9') or 
                 (This.Current_Character = '_') or 
                 (This.Current_Character in 'a'..'f')  
              then
                 Save_Character;
              else
                 exit;
              end if;
           end loop;
         end Get_Extended_Integer;

         
      begin -- GET NUMERIC LITERAL

         Token.Lexical_Unit := Numeric_Literal_Lex;
         Token.Lexicon      := Blank_Identifier;
         Save_Character;

         Get_Integer;

         if This.Current_Character = '.' then
            Save_Character;
            Get_Integer;
            if This.Current_Character = 'E' then
               Save_Character;
               Get_Exponent;
            end if;

         elsif This.Current_Character = '#' then
            Save_Character;
            Get_Extended_Integer;
            if This.Current_Character = '.' then
               Save_Character;
               Get_Extended_Integer;
            end if;
            if This.Current_Character = '#' then
               Save_Character;
               Get_Exponent;
            end if;       
         end if;     
      end Get_Numeric_Literal;


      -- GET CHARACTER LITERAL -------------------------------------

      procedure Get_Character_Literal is
      --|Description>
      --|  Procedure which searches for character
      --|  literals and then calls the Save_Character
      --|  procedure to save them as their lexicon
      --|  values (integer string values).
      --||Description>
      begin

         Token.Lexical_Unit := Character_Literal_Lex;
         Token.Lexicon      := Blank_Identifier;
         Save_Character;

         -- Check for special character literals '''

         if (This.Line_Pointer) <= This.Last_Character_In_Line then
            if This.Input_Line(This.Line_Pointer) = ''' then
               Save_Character;
               Save_Character;
            else
               Token.Lexical_Unit := Delimiter_Lex;
            end if;
         else
             Token.Lexical_Unit := Delimiter_Lex;
         end if;            

      end Get_Character_Literal;


      -- GET STRING -----------------------------------------------
      procedure Get_String is
      --|Description>
      --|  Procedure which searches for strings and then
      --|  calls the Save_Character procedure to save them
      --|  as their lexicon values (integer string values).
      --||Description>
      begin
         Token.Lexical_Unit := String_Lex;
         Token.Lexicon      := Blank_Identifier;
         Save_Character;

         -- Add chars until """ " or """ or """" found
         loop
            if This.Current_Character = '"' then
               Save_Character;
               if This.Current_Character = '"' then
                  Save_Character;
               else
                  exit;
               end if;
            else
               Save_Character;
            end if;
         end loop;
      end Get_String;


      -- GET COMMENT --------------------------------------------------
      procedure Get_Comment is
      --|Description>
      --|  Procedure which identifies comments and then
      --|  bypasses them by going to the next line of
      --|  code.
      --||Description>
         item : List_Element.Handle := null;
      begin
         Token.Lexical_Unit := Comment_Lex;
         Token.Lexicon      := Blank_Identifier;
         Save_Character;

         if This.Current_Character = '-' then   
	 
	    NextCH(This);
	    if This.Current_Character = '|' then	           	
		Add( This.Comments,
		     This.Input_Line(This.Line_Pointer..This.Last_Character_In_Line)
		     );
	    end if;
            -- If found then trash the rest of the line
            This.Line_Pointer := String_Line_Range_Type'last;         
         else
            Token.Lexical_Unit := Delimiter_Lex;
         end if;
      end Get_Comment;


      -- GET DELIMITER -----------------------------------------------
      procedure Get_Delimiter is
      --|Description>
      --|  Procedure which identifies specific delimiters
      --|  and calls the Save_Character procedure to save
      --|  them as their lexicon values (integer string
      --|  values).
      --||Description>
      begin

         Token.Lexical_Unit := Delimiter_Lex;
         Token.Lexicon      := Blank_Identifier;

         -- Check for the single and double operators

         case This.Current_Character is
            when '=' =>
               Save_Character;
               if This.Current_Character = '>' then
                  Save_Character;
               end if;
            when '.' =>
               Save_Character;
               if This.Current_Character = '.' then
                  Save_Character;
               end if;
            when '*' =>
               Save_Character;
               if This.Current_Character = '*' then
                  Save_Character;
               end if;
            when ':' =>
               Save_Character;
               if This.Current_Character = '=' then
                  Save_Character;
               end if;
            when '/' =>
               Save_Character;
               if This.Current_Character = '=' then
                  Save_Character;
               end if;
            when '>' =>
               Save_Character;
               if (This.Current_Character = '=') or (This.Current_Character = '>') then
                  Save_Character;
               end if;
            when '<' =>
               Save_Character;
               if (This.Current_Character = '=') or (This.Current_Character = '<') or (This.Current_Character = '>') then
                  Save_Character;
               end if;
            when '&' | ''' | '(' | ')' | '+' | ',' | '-' | ';' | '|' =>
               Save_Character;
            when ASCII.EOT =>
               Token.Lexical_Unit := End_Of_File_Lex;
            when others =>
               Token.Lexical_Unit := Unknown_Lex;
               Save_Character;
         end case;
      end Get_Delimiter;


   begin -- GET TOKEN

      -- Skip white space
      while (This.Current_Character = Space) or (This.Current_Character = Tab) loop 
         NextCh(This);
      end loop;

      -- The first character determines token type
      case This.Current_Character is 

         when 'a'..'z' =>
            Get_Identifier;

         when '0'..'9'            =>
            Get_Numeric_Literal;

         when Latin_1.Apostrophe  =>
            Get_Character_Literal;

         when Latin_1.Quotation   =>     
            Get_String;

         when '-'                 =>
            Get_Comment;

         when others              =>
            Get_Delimiter;

      end case; 
   end Get_Token;

-- GET NEXT SYMBOL -------------------------------------------------------------

   procedure Get_Next_Symbol( This : in Object_Data_access ) is
      --|Description>
      --|  This procedure gets the next symbol or token, identifying
      --|  its type (delimiter, reserved word identifier, numeric,
      --|  string, character, or unknown).
      --||Description>

      function Equal( S1 : String; S2 : String ) return Boolean is
         --|Description>
         --|  This function checks to see if two strings are equal.  It
         --|  is used by the procedures Find_Identifier_Type and 
         --|  Find_Delimiter_Type when attempting to match current symbols
         --|  with values in predefined reserved word and delimiter tables.
         --||Description>
      begin
         for I in S1'Range loop
            if S1(I..I) = " " then
               if S2(I..I) = " " then
                  return True;
               else 
                  return False;
               end if;
            elsif S1(I..I) /= S2(I..I) then
               return False;
            end if; 
         end loop;

         return False;
      end Equal;

      procedure Find_Identifier_Type( Symbol  : out Ada_Symbol_Type ) is
         --|Description>
         --|  This procedure searches for an identifier within
         --|  a reserved word table.
         --||Description>
         Index  : Reserved_Word_Range_Type := Abort_Symbol;
      begin

         -- Simple sequential search to separate reserved words from identifiers
         Reserved_Word_Table(Identifier_Symbol) := This.Next_Token.Lexicon;
         loop
            if Equal ( Reserved_Word_Table(Index), This.Next_Token.Lexicon ) then
               exit;
            else
               Index := Reserved_Word_Range_Type'Succ(Index);
            end if;
         end loop;

         Symbol := Index;

      end Find_Identifier_Type;

      procedure Find_Delimiter_Type( Symbol  : out Ada_Symbol_Type ) is
         --|Description>
         --|  This procedure searches for an identifier within a
         --|  delimiter table.
         --||Description>
         Index  : Delimiter_Range_Type := Equal_Symbol;
      begin

         -- Simple sequential search to separate UNKNOWNS from operators
         Delimiter_Table(Unknown_Symbol) := This.Next_Token.Lexicon;
         loop
            if Equal ( Delimiter_Table(Index), This.Next_Token.Lexicon ) then
               exit;
            else
               Index := Delimiter_Range_Type'Succ(Index);
            end if;
         end loop;

         Symbol := Index;
      end Find_Delimiter_Type;


   begin -- GET NEXT TOKEN 

      -- Loop to trash comments only
      loop
         Get_Token( This, This.Next_Token );

         case This.Next_Token.Lexical_Unit is
            when Identifier_Lex        =>
               Find_Identifier_Type( This.Next_Ada_Symbol );

               -- Trash pragmas as well
               if This.Next_Ada_Symbol = Pragma_Symbol then
                  This.First_Pragma_Identifier := True;
                  loop
                     if This.Next_Ada_Symbol = Semicolon_Symbol then
                        Get_Next_Symbol(This);
                        exit;
                     elsif This.Next_Ada_Symbol = Unknown_Symbol then
                        exit;
                     else
                        if This.Next_Ada_Symbol = Identifier_Symbol then
                           if This.First_Pragma_Identifier then
                              -- Pragma_Encountered( O, Next_Token.Lexicon);
                              This.First_Pragma_Identifier := False;
                           end if;
                        end if;
                        Get_Next_Symbol(This);
                     end if;
                  end loop;
               elsif  This.Next_Ada_Symbol = For_Symbol then
                  loop
                     Get_Next_Symbol(This);
                     if This.Next_Ada_Symbol = In_Symbol then
                        exit;
                     elsif This.Next_Ada_Symbol = Use_Symbol then
                        Get_Next_Symbol(This);
                        if This.Next_Ada_Symbol = At_Symbol then
                           -- PD.Address_Representation_Clause_Encountered;
                           null;
                        else
                           -- PD.Type_Representation_Clause_Encountered;
                           null;
                        end if;
                        exit;
                     end if;
                  end loop;
               end if;
               exit;

            when Character_Literal_Lex =>
               This.Next_Ada_Symbol := Character_Symbol;
               exit;

            when Comment_Lex           =>
               Get_Token( This, This.Next_Token );

            when Numeric_Literal_Lex   =>
               This.Next_Ada_Symbol := Numeric_Symbol;
               exit;

            when String_Lex            =>
               This.Next_Ada_Symbol := String_Symbol;
               exit;

            when Delimiter_Lex         =>
               Find_Delimiter_Type( This.Next_Ada_Symbol );
               exit;

            when others                   =>
               This.Next_Ada_Symbol := Unknown_Symbol;
               exit;
         end case;

      end loop;

      This.Current_Symbol := This.Next_Ada_Symbol;
      This.Current_Token  := This.Next_Token;

   end Get_Next_Symbol;

-- SKIP NEXT SYMBOL ---------------------------------------------------

   procedure Skip_Next_Symbol(This : in Object_Data_access ) is
      --|Description>
      --|  This procedure bit-buckets the next symbol and
      --|  then calls the Get_Next_Symbol(This) procedure.
      --||Description>
   begin
      Get_Next_Symbol( This );
   end Skip_Next_Symbol;
  

-- ACCEPT SYMBOL --------------------------------------------------------------
   procedure Accept_Symbol ( This : in Object_Data_access;
                             Expected_Symbol : in Ada_Symbol_Type ) is
      --|Description>
      --|  This procedure checks to see if the Expected_Symbol
      --|  is equal to the Current_Symbol.  If they are not equal,
      --|  the exception Syntax_Error is raised.
      --||Description>
   begin
      if Expected_Symbol = This.Current_Symbol then
         Get_Next_Symbol(This);
      else
         raise Syntax_Error;
      end if;
   end Accept_Symbol;


-- GET CURRENT SYMBOL -------------------------------------------------

   function Get_Current_Symbol(This : in Object_Data_access ) return Ada_Symbol_Type is
      --|Description>
      --|  This function returns to the caller the current symbol.
      --||Description>
   begin
      return This.Current_Symbol;
   end Get_Current_Symbol;

   
-- GET CURRENT TOKEN -------------------------------------------------

   function Get_Current_Token(This : in Object_Data_access) return Token_Type is
      --|Description>
      --|  This function returns to the caller the current lexical 
      --|  token, which is a record of the lexical unit type and
      --|  its integer string value.
      --||Description>
   begin
      return This.Current_Token;
   end Get_Current_Token;

   
-- GET CURRENT LEXICON -----------------------------------------------

   function Get_Current_Lexicon(This : in Object_Data_access ) 
                                           return Simple_Identifier_Type is
      --|Description>
      --|  This function returns to the caller the integer string
      --|  value for the current lexical token.
      --||Description>
   begin
      return This.Current_Token.Lexicon;
   end Get_Current_Lexicon;


-- SKIP SYMBOLS UNTIL TERMINATOR FOUND --------------------------------

   procedure Skip_Symbols_Until_Terminator_Found( 
                 This              : in Object_Data_access;
                 Valid_Terminators : in Ada_Symbol_Set_Type ) is
      --|Description>
      --|  This procedure checks to see if the current
      --|  symbol is a member of the desired symbol set;
      --|  if so, the procedure exits, else, it gets the 
      --|  next symbol.
      --||Description>
   begin
      loop
         if Valid_Terminators(This.Current_Symbol) then
            exit;
         else
            Get_Next_Symbol(This);
         end if;
      end loop;
   end Skip_Symbols_Until_Terminator_Found;

   --|
   --| Copy all till an valid terminator is found and return the
   --| result string
   --|
   function Copy_Until_Terminator_Found( 
                 This              : in Object_Data_access;
                 Valid_Terminators : in Ada_Symbol_Set_Type ) 
		                          return Unbounded_String 
   is
      Result : Unbounded_String := To_Unbounded_String("");
   begin
      loop
         if Valid_Terminators(This.Current_Symbol) then
            exit;
         else
	    Result := Result & " " & Trim( Get_Current_Lexicon(This), Right);
            Get_Next_Symbol(This);
         end if;
      end loop;

      return Result;
   end Copy_Until_Terminator_Found;


   -- CLOSE ADA SOURCE FILE ----------------------------------------------

   procedure Close_Ada_Source_File(This : in Object_Data_access) is
   begin
      if This.File_Is_Open then
         TI.Close( This.Input_File );
         This.File_Is_Open := False;
      end if;
   end Close_Ada_Source_File;


   -- OPEN NEW ADA SOURCE FILE --------------------------------------------

   procedure Open_New_Ada_Source_File( O       : in Object'Class;
                                       Ada_Src : in String ) is
   --|Description>
   --|  This procedure opens a new ada source file, puts a 
   --|  message to the user's screen saying that the program
   --|  is now parsing this file (Parsing> "filename"), and
   --|  performs initialization functions.
   --||Description>
      This : Object_Data_Access := O.Data;
   begin
      -- First close the last file
      Close_Ada_Source_File(This);

      TI.Open(File => This.Input_File,
              Mode => TI.In_File,
              Name => Ada_Src );
      This.File_Is_Open := True;

      -- Initialize stuff
      This.Current_Character      := Space;
      This.Line_Pointer           := String_Line_Range_Type'last;
      This.Last_Character_In_Line := 1; 
      This.Current_Line_Count     := 0;

      -- NOTE: this willl restart the scanner module
      Skip_Next_Symbol(This); 

      -- PD.New_File_Opened;

      Trace( This.Debug, "Parsing> ");
      Trace( This.Debug,  Ada_Src );

   exception

      when TI.Status_Error =>
         Trace( This.Debug, "<<<Error>>> File already open: ");
         Trace( This.Debug, Ada_Src);
         This.File_Is_Open := False;

      when TI.Name_Error =>
         Trace( This.Debug, "<<<Error>>> File name error or file not found: ");
         Trace( This.Debug, Ada_Src);
         This.File_Is_Open := False;

      when TI.Use_Error =>
         Trace( This.Debug, "<<<Error>>> File use error: ");
         Trace( This.Debug, Ada_Src);
         This.File_Is_Open := False;

      when others          =>
         Trace( This.Debug, "<<<Error>>> Unknown file open error on input file name: ");
         Trace( This.Debug, Ada_Src);
         This.File_Is_Open := False;

   end Open_New_Ada_Source_File;


-- PARSE CONTEXT CLAUSE ------------------------------------------------------------

   procedure Parse_Context_Clause( O : in out Object'Class ) is 
      --|Description>
      --|  The context clause of the current input Ada source file is 
      --|  parsed.  With and use identifiers are declared.
      --||Description>
      
      This       : Object_Data_Access := O.Data;
      Use_Found  : Boolean := True;
      With_Found : Boolean := True;
      Lib_Name   : Unbounded_String := To_Unbounded_String("");

      procedure Accept_Library_Name is
      begin
     	 Lib_Name := To_Unbounded_String( 
	                 Trim(Get_Current_Lexicon(This),Right) );

         Accept_Symbol( This, Identifier_Symbol );
         while Get_Current_Symbol(This) = Period_Symbol loop

            Accept_Symbol( This, Period_Symbol );
 
	    Lib_Name := Lib_Name & "." & 
	                 To_Unbounded_String( 
			     Trim(Get_Current_Lexicon(This), Right));
 
            Accept_Symbol( This, Identifier_Symbol );
         end loop;
      end Accept_Library_Name;

      procedure Parse_With_Clause( Found : out Boolean ) is
         --|Description>
         --|  The with clause of the current input Ada source file is 
         --|  parsed.  
         --||Description>
      begin
         if Get_Current_Symbol(This) /= With_Symbol then
            Found := False;
         else
            Found := True;
            Accept_Symbol( This, With_Symbol );

            Accept_Library_Name;
	    Add( This.Libraries, To_String(Lib_Name) );
            loop
               if Get_Current_Symbol(This) /= Comma_Symbol then
                  exit;
               else
                  Accept_Symbol( This, Comma_Symbol );
                  Accept_Library_Name;
               end if;
            end loop;
            Accept_Symbol( This, Semicolon_Symbol );
         end if;
      exception
         when Syntax_Error =>
            Found := False;
      end Parse_With_Clause;


      procedure Parse_Use_Clause( Found : out Boolean ) is
         --|Description>
         --|  The with clause of the current input Ada source file is 
         --|  parsed.  
         --||Description>
      begin
         if Get_Current_Symbol(This) /= Use_Symbol then
            Found := False;
         else
            Found := True;
            Accept_Symbol( This, Use_Symbol );
            Accept_Library_Name;
            loop
               if Get_Current_Symbol(This) /= Comma_Symbol then
                  exit;
               else
                  Accept_Symbol( This, Comma_Symbol );
                  Accept_Library_Name;
               end if;
            end loop;
            Accept_Symbol( This, Semicolon_Symbol );
         end if;
      exception
         when Syntax_Error =>
            Found := False;
      end Parse_Use_Clause;

   begin
      while With_Found loop
         Parse_With_Clause( With_Found );
         if With_Found then
            Use_Found := True;
            while Use_Found loop
               Parse_Use_Clause( Use_Found );
            end loop;
         end if;
      end loop;
   end Parse_Context_Clause;
   
   procedure Parse_Compilation_Unit( O : in out Object'Class ) is
      --|Description>
      --|  Procedure which parses an Ada compilation unit.  This is a
      --|  recursive descent compiler (brain damaged).  It is separate
      --|  because of its size.
      --||Description>

      This		 : Object_Data_Access := O.Data;
      Current_Token      : Token_Type;
      Found              : Boolean             := False;
      Name               : Simple_Identifier_Type;
      Un_Named_Exception : Simple_Identifier_Type 
        := "UNNAMED EXCEPTION                               ";

      Pkg_Name		 : Unbounded_String := Null_Unbounded_String;

      procedure Parse_Ada_Name( Identifier : out Simple_Identifier_Type );
      --|Description>
      --|  Procedure which parses an Ada name.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Body;
      --|Description>
      --|  Procedure which parses Ada bodies for major symbols
      --|  (Accept, Function, Generic, Package, Procedure, Begin, 
      --|  etc.) and then calls appropriate routines to further
      --|  parse the symbols.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Function_Symbol;
      --|Description>
      --|  Procedure which parses the Ada function symbol.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Generic_Symbol;
      --|Description>
      --|  Procedure which parses the Ada generic symbol.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Package_Specification;
      --|Description>
      --|  Procedure which parses Ada specifications for major
      --|  symbols (Function, Generic, Package, Procedure, 
      --|  Private, Task, Case, etc.) and then calls the appropriate
      --|  routines to further parse the symbols. 
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Package_Symbol;
      --|Description>
      --|  Procedure which parses the Ada package symbol.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Procedure_Symbol;
      --|Description>
      --|  Procedure which parses the Ada procedure symbol.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


      procedure Parse_Task_Symbol;
      --|Description>
      --|  Procedure which parses the Ada task symbol.
      --|  Note:  This is a recursive forward
      --|         declaration.
      --||Description>


-- PARSE ACCEPT_SYMBOL ---------------------------------------------------------

      procedure Parse_Accept_Symbol is
         --|Description>
         --|  Procedure which parses the Ada accept symbol.
         --||Description>
         Name : Simple_Identifier_Type;
      begin
         Accept_Symbol( This, Accept_Symbol );
         Parse_Ada_Name( Name );
         Skip_Symbols_Until_Terminator_Found( This, 
                                              Accept_Statement_Terminators );
         if Get_Current_Symbol(This) = Do_Symbol then
            Accept_Symbol( This, Do_Symbol );
            Parse_Body;
         end if;
      end Parse_Accept_Symbol;


-- PARSE ADA NAME --------------------------------------------------------------

      procedure Parse_Ada_Name( Identifier : out Simple_Identifier_Type ) is
         --|Description>
         --|  Procedure which parses an Ada name.
         --||Description>
         Found : Boolean             := False;

      begin
         while Get_Current_Symbol(This) = Identifier_Symbol loop
            Identifier := Get_Current_Lexicon(This);
            Accept_Symbol(This, Identifier_Symbol );
            if Get_Current_Symbol(This) = Period_Symbol then
               Skip_Next_Symbol(This);
            end if;         
         end loop;
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap Parse_Ada_Name <DBG>");
      end Parse_Ada_Name;


-- PARSE TYPE MARK -------------------------------------------------------------

      procedure Parse_Type_Mark is
         --|Description>
         --|  Procedure which parses the Ada type mark.
         --||Description>
      begin
         This.Return_Type := Copy_Until_Terminator_Found( 
                                This,
                                Type_Mark_Terminators );                                          
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap Parse_Type_Mark <DBG>");
      end Parse_Type_Mark;


-- PARSE FORMAL PARAMETERS -----------------------------------------------------

      procedure Parse_Formal_Parameters is
         --|Description>
         --|  Procedure which parses Ada formal parameters.
         --||Description>
         Parens : Integer := 0;

      begin
         if Get_Current_Symbol(This) = LPara_Symbol then
            Accept_Symbol( This, LPara_Symbol );
            Parens := Parens + 1;
            loop

               This.Args := Copy_Until_Terminator_Found( 
                                This,
                                Formal_Parameter_Terminators );

               if Get_Current_Symbol(This) = RPara_Symbol then
                  Accept_Symbol( This, RPara_Symbol );
                  Parens := Parens - 1;
                  if Parens <= 0 then
                     exit;
                  end if;
               else
                  Accept_Symbol( This, LPara_Symbol );
                  Parens := Parens + 1;
               end if;
            end loop;
         end if;
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap Parse_Formal_Parameters <DBG>");
      end Parse_Formal_Parameters;


-- PARSE BODY ---------------------------------------------------------------------

      procedure Parse_Body is
         --|Description>
         --|  Procedure which parses Ada bodies for major symbols
         --|  (Accept, Function, Generic, Package, Procedure, Begin, 
         --|  etc.) and then calls appropriate routines to further
         --|  parse the symbols.
         --||Description>
         Begin_Block     : Boolean := False;
         Current_Lexicon : Simple_Identifier_Type;
         Found           : Boolean := False;
         Nested_Count    : Integer := 1;
       begin 
         loop
            Skip_Symbols_Until_Terminator_Found( This, Body_Starters );
            case Get_Current_Symbol(This) is
               when Accept_Symbol =>
                  Parse_Accept_Symbol;
               when Function_Symbol =>
                  Parse_Function_Symbol;
               when Generic_Symbol =>
                  Parse_Generic_Symbol;
               when Package_Symbol =>
                  Parse_Package_Symbol;
               when Procedure_Symbol =>
                  Parse_Procedure_Symbol;
               when Task_Symbol =>
                  Parse_Task_Symbol;
               when Case_Symbol | Record_Symbol | Loop_Symbol | 
                 If_Symbol | Select_Symbol =>
                  Skip_Next_Symbol(This);
                  Nested_Count := Nested_Count + 1;
               when Begin_Symbol =>
                  Skip_Next_Symbol(This);
                  if Begin_Block then
                     Nested_Count := Nested_Count + 1;
                  else
                     Begin_Block := True;
                  end if;
               when End_Symbol =>
                  Nested_Count := Nested_Count - 1;
                  Accept_Symbol( This, End_Symbol );
                  if Nested_Count <= 0 then
                     if Get_Current_Symbol(This) = Identifier_Symbol then
                        Accept_Symbol( This, Identifier_Symbol );
                     end if;
                     Skip_Next_Symbol(This);
                     exit;
                  else
                     Skip_Next_Symbol(This);
                  end if;
               when Raise_Symbol =>
                  Accept_Symbol( This, Raise_Symbol );
                  if Get_Current_Symbol(This) = Identifier_Symbol then
                     Parse_Ada_Name( Current_Lexicon );
                     -- Exception_Raised( O, Current_Lexicon );
                  else
                     -- Exception_Raised( O, Un_Named_Exception );
                     null;
                  end if;
               when others => -- Identifier_Symbol 
                  Skip_Next_Symbol(This);
            end case;
         end loop;
         Exit_Scope(O);
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap in Parse_Body <DBG>");
      end Parse_Body;


-- PARSE FUNCTION SYMBOL ----------------------------------------------------------
      procedure Parse_Function_Symbol is
         --|Description>
         --|  Procedure which parses the Ada function symbol.
         --||Description>

         Current_Lexicon : Simple_Identifier_Type;
	 Specification   : Contents;
	 item            : List_Element.Handle := null;

      begin
         Accept_Symbol( This, Function_Symbol );
         if Get_Current_Symbol(This) = Identifier_Symbol or
            Get_Current_Symbol(This) = String_Symbol then

            Current_Lexicon := Get_Current_Lexicon(This);
	    Specification.Name := To_Unbounded_String(Current_Lexicon);

            Skip_Next_Symbol(This);
            Parse_Formal_Parameters;
            if Get_Current_Symbol(This) = Return_Symbol then
               Accept_Symbol( This, Return_Symbol );
               Parse_Type_Mark;
            end if;
            Enter_Scope( O, Current_Lexicon );
            if Get_Current_Symbol(This) = Is_Symbol then
               Accept_Symbol( This, Is_Symbol );
               if Get_Current_Symbol(This) = Separate_Symbol then
                  Accept_Symbol( This, Separate_Symbol );
                  Accept_Symbol( This, Semicolon_Symbol );
                  Exit_Scope(O);
	       elsif Get_Current_Symbol(This) = Abstract_Symbol then
                  Accept_Symbol( This, Abstract_Symbol );
                  Accept_Symbol( This, Semicolon_Symbol );

	          Specification.Is_Abstract := True;
                  Exit_Scope(O);	       
               else
                  if Get_Current_Symbol(This) = New_Symbol then
                     Accept_Symbol( This, New_Symbol );
                     Exit_Scope(O);
                  else
                     Parse_Body;
                  end if;
               end if;
            else
               Accept_Symbol( This, Semicolon_Symbol );
               Exit_Scope(O);
            end if;

	    Specification.What      := Method;
	    Specification.Arguments := this.Args;
	    Specification.Returns   := this.Return_Type;
	    Specification.Comments  := Text(this.Comments);

	    item := Construct( Specification );
	    Append( This.Result, Item.all );

	    Finalize( this.Comments );
	    Initialize( this.Comments );

         end if;
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap Parse_Function_Symbol <DBG>");
      end Parse_Function_Symbol;

-- PARSE GENERIC SYMBOL -----------------------------------------------------------

      procedure Parse_Generic_Symbol is
         --|Description>
         --|  Procedure which parses the Ada generic symbol.
         --||Description>
      begin

         Accept_Symbol( This, Generic_Symbol );
         loop
            Skip_Symbols_Until_Terminator_Found( This, Generic_Starters );
            case Get_Current_Symbol(This) is
               when Function_Symbol  =>
                  Parse_Function_Symbol;
                  exit;
               when Package_Symbol   =>
                  Parse_Package_Symbol;
                  exit;
               when Procedure_Symbol =>
                  Parse_Procedure_Symbol;
                  exit;
               when others              =>
                  Accept_Symbol( This, With_Symbol );
                  Skip_Next_Symbol(This);
            end case;
         end loop;

      end Parse_Generic_Symbol;


-- PARSE PACKAGE SPECIFICATION ----------------------------------------------------

      procedure Accept_Package_Name is
      begin
         Pkg_Name := To_Unbounded_String(Trim(Get_Current_Lexicon(This),Right));

         Accept_Symbol( This, Identifier_Symbol );
         while Get_Current_Symbol(This) = Period_Symbol loop
            Accept_Symbol( This, Period_Symbol );

            Pkg_Name := Pkg_Name & "." & 
                            To_Unbounded_String( 
                                 Trim(Get_Current_Lexicon(This), Right));

            Accept_Symbol( This, Identifier_Symbol );
         end loop;
      end Accept_Package_Name;


      procedure Parse_Package_Specification is
         --|  Procedure which parses Ada specifications for major
         --|  symbols (Function, Generic, Package, Procedure, 
         --|  Private, Task, Case, etc.) and then calls the appropriate
         --|  routines to further parse the symbols. 
         --||Description>

         Done : Boolean := False;

         procedure Parse_Basic_Declarative_Item is
            Nested_End_Count : Integer := 0;
         begin
            Done := False;
            loop
               Skip_Symbols_Until_Terminator_Found( This,
                                                    Package_Spec_Starters );
               case Get_Current_Symbol(This) is
                  when Function_Symbol =>
                     Parse_Function_Symbol;
                     Done := True;
                     exit;
                  when Generic_Symbol =>
                     Parse_Generic_Symbol;
                     Done := True;
                     exit;
                  when Package_Symbol =>
                     Parse_Package_Symbol;
                     Done := True;
                     exit;
                  when Procedure_Symbol =>
                     Parse_Procedure_Symbol;
                     Done := True;
                     exit;
                  when Private_Symbol =>
                     exit;
                  when Task_Symbol =>
                     Parse_Task_Symbol;
                     Done := True;
                     exit;

                  when Null_Symbol =>
                     if Nested_End_Count > 0 then
                        Skip_Next_Symbol(This);
                        exit;
                     end if;
                  when Case_Symbol | Record_Symbol  =>
                     Nested_End_Count := Nested_End_Count + 1;
                     Skip_Next_Symbol(This);
                  when others =>  -- End_Symbol =>
                     if Nested_End_Count > 0 then
                        Nested_End_Count := Nested_End_Count - 1;
                        Accept_Symbol( This, End_Symbol );
                        if Get_Current_Symbol(This) = Case_Symbol then
                           Skip_Next_Symbol(This);
                        elsif Get_Current_Symbol(This) = Record_Symbol then
                           Skip_Next_Symbol(This);
                        else 
                           Skip_Next_Symbol(This);
                        end if;
                     else
                        exit;
                     end if;
               end case;
            end loop;
         exception 
            when others =>
               Trace( This.Debug, "<DBG> others trap Parse_Basic_Declarative_Item <DBG>");
	       Trace( This.Debug, This.Input_Line);
         end Parse_Basic_Declarative_Item;


      begin -- PARSE PACKAGE SPECIFICATION
         loop
            Parse_Basic_Declarative_Item;
            if not Done then
               exit;
            end if; 
         end loop;
         if Get_Current_Symbol(This) =  Private_Symbol then
            Accept_Symbol( This, Private_Symbol );
            Done := False;
            loop
               Parse_Basic_Declarative_Item;
               if not Done then
                  exit;
               end if; 
            end loop;
         end if;
         Accept_Symbol( This, End_Symbol );
         -- Accept_Symbol( This, Identifier_Symbol );
	 Accept_Package_Name;
         Accept_Symbol( This, Semicolon_Symbol );
      exception
         when Syntax_Error =>
            Trace( This.Debug, "<Warning> Syntax error in Parse_Package_Specification <Warning>");
            Trace( This.Debug, This.Input_Line);
         when others =>
            Trace( This.Debug, "<DBG> others trap Parse_Package_Specification <DBG>");
      end Parse_Package_Specification;

      procedure Parse_Package_Symbol is
         --|Description>
         --|  Procedure which parses the Ada package symbol.
         --||Description>

         Current_Lexicon : Simple_Identifier_Type;
	 Specification   : Contents;
	 item            : List_Element.Handle := null;

      begin
         Accept_Symbol(This, Package_Symbol );
         if Get_Current_Symbol(This) = Body_Symbol then

            Accept_Symbol( This, Body_Symbol );
            Current_Lexicon := Get_Current_Lexicon(This);
            Accept_Symbol( This, Identifier_Symbol );
            Enter_Scope ( O, Current_Lexicon );
            Accept_Symbol( This, Is_Symbol );
            if Get_Current_Symbol(This) = Separate_Symbol then
               Accept_Symbol( This, Separate_Symbol );
               Accept_Symbol( This, Semicolon_Symbol );
               Exit_Scope(O);
            else
               Parse_Body;
            end if;

         else
            -- Current_Lexicon := Get_Current_Lexicon(This);
            -- Accept_Symbol( This, Identifier_Symbol );
	    Accept_Package_Name;

            if Get_Current_Symbol(This)  = Renames_Symbol then
               Accept_Symbol( This, Renames_Symbol );
               while Get_Current_Symbol(This) /= Semicolon_Symbol loop
                  Skip_Next_Symbol(This);
               end loop;
               Accept_Symbol( This, Semicolon_Symbol );
            else
               Enter_Scope( O, Current_Lexicon ); 
	        
	       Specification.Name      := Pkg_Name;

	       Specification.What      := Package_Unit;
	       Specification.Comments  := Text(this.Comments);
	       Specification.Libraries := Text(This.Libraries);

	       item := Construct( Specification );
	       Append( This.Result, Item.all );

	       Finalize( this.Comments );
	       Initialize( this.Comments );

	       -- continue processing 
               Accept_Symbol( This, Is_Symbol );
               if Get_Current_Symbol(This) = New_Symbol then
                  Accept_Symbol( This, New_Symbol );
               else
                  Parse_Package_Specification;
               end if;
               Exit_Scope(O);
            end if;
         end if;
      exception 
         when others =>
            Break("<DBG> others trap in Parse_Package_Symbol <DBG>");
	    Break(This.Input_Line);

      end Parse_Package_Symbol;


-- PARSE PROCEDURE SYMBOL ----------------------------------------------------------

      procedure Parse_Procedure_Symbol is
         --|Description>
         --|  Procedure which parses the Ada procedure symbol.
         --||Description>

         Current_Lexicon : Simple_Identifier_Type;
	 Specification   : Contents;
	 item            : List_Element.Handle := null;
      begin
         Accept_Symbol( This, Procedure_Symbol );

         Current_Lexicon := Get_Current_Lexicon(This);
         if Get_Current_Symbol(This) = Identifier_Symbol then
            Parse_Ada_Name( Current_Lexicon );
	    Specification.Name := To_Unbounded_String( Current_Lexicon );
         end if;
         -- Accept_Symbol( Identifier_Symbol );

         Parse_Formal_Parameters;

         Enter_Scope( O, Current_Lexicon );
         if Get_Current_Symbol(This) = Is_Symbol then
            Accept_Symbol( This, Is_Symbol );

            if Get_Current_Symbol(This) = Separate_Symbol then
               Accept_Symbol( This, Separate_Symbol );
               Accept_Symbol( This, Semicolon_Symbol );
               Exit_Scope(O);
	    elsif Get_Current_Symbol(This) = Abstract_Symbol then
               Accept_Symbol( This, Abstract_Symbol );
               Accept_Symbol( This, Semicolon_Symbol );

	       Specification.Is_Abstract := True;
               Exit_Scope(O);	       
            else
               if Get_Current_Symbol(This) = New_Symbol then
                  Accept_Symbol(This, New_Symbol );
                  Exit_Scope(O);
               else
                  Parse_Body;
               end if;
            end if;
         else
            Accept_Symbol( This, Semicolon_Symbol );
            Exit_Scope(O);
         end if;

	 Specification.What      := Method;
	 Specification.Arguments := this.Args;
	 Specification.Comments  := Text(this.Comments);

	 item := Construct( Specification );
	 Append( This.Result, Item.all );

	 Finalize( this.Comments );
	 Initialize( this.Comments );

      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap in Parse_Procedure_Symbol <DBG>");
      end Parse_Procedure_Symbol;

-- PARSE TASK SYMBOL ----------------------------------------------------------

      procedure Parse_Task_Symbol is
         --|Description>
         --|  Procedure which parses the Ada task symbol.
         --||Description>
         Current_Lexicon : Simple_Identifier_Type;


         procedure Parse_Task_Specification is

            --|Description>
            --|  Proceduew which parses the Ada task specification.
            --||Description>

         begin
            Current_Lexicon := Get_Current_Lexicon(This);
            Enter_Scope( O, Current_Lexicon );
            Accept_Symbol(This, Identifier_Symbol );
            if Get_Current_Symbol(This) = Is_Symbol then
               Accept_Symbol(This, Is_Symbol );
               loop
                  Skip_Symbols_Until_Terminator_Found( This,
                                                       Task_Spec_Starters );
                  case Get_Current_Symbol(This) is
                     when Entry_Symbol =>
                        Accept_Symbol( This, Entry_Symbol );
                        Enter_Scope( O, Get_Current_Lexicon(This) );
                        Accept_Symbol( This, Identifier_Symbol );
                        Exit_Scope(O);
                     when others          =>
                        Accept_Symbol( This, End_Symbol );
                        if Get_Current_Symbol(This) = Identifier_Symbol then
                           Accept_Symbol( This, Identifier_Symbol );
                        end if;
                        Exit_Scope(O);
                        exit;
                  end case;
               end loop;
            end if;
            -- PD.Exit_Scope;
         end Parse_Task_Specification;

      begin
         Accept_Symbol( This, Task_Symbol );
         case Get_Current_Symbol(This) is
            when Body_Symbol       =>
               Accept_Symbol( This, Body_Symbol );
               Enter_Scope( O, Get_Current_Lexicon(This) );
               Accept_Symbol( This, Identifier_Symbol );
               Accept_Symbol( This, Is_Symbol );
               if Get_Current_Symbol(This) = Separate_Symbol then
                  Accept_Symbol( This, Separate_Symbol );
                  Accept_Symbol( This, Semicolon_Symbol );
                  -- PD.Exit_Scope;
               else
                  Parse_Body;
               end if;
            when Identifier_Symbol =>
               Parse_Task_Specification;
            when others               =>
               Accept_Symbol( This, Type_Symbol );
               Parse_Task_Specification;
         end case;
      exception 
         when others =>
            Trace( This.Debug, "<DBG> others trap in Parse_Task_Symbol <DBG>");
      end Parse_Task_Symbol;


   begin
      case Get_Current_Symbol(This) is
         when Generic_Symbol =>
            Parse_Generic_Symbol;
         when Package_Symbol =>
            Parse_Package_Symbol;
         when Procedure_Symbol =>
            Parse_Procedure_Symbol;
         when Function_Symbol =>
            Parse_Function_Symbol;
         when Separate_Symbol =>
            Accept_Symbol( This, Separate_Symbol );
            Accept_Symbol( This, LPara_Symbol );
            Parse_Ada_Name( Name );
            Accept_Symbol( This, RPara_Symbol );
            case Get_Current_Symbol(This) is
               when Package_Symbol =>
                  Parse_Package_Symbol;
               when Procedure_Symbol =>
                  Parse_Procedure_Symbol;
               when Function_Symbol =>
                  Parse_Function_Symbol;
               when Task_Symbol =>
                  Parse_Task_Symbol;
               when others =>
                  Trace( This.Debug, "<Error> Unexpected symbol encountered <Error>");

            end case;
         when others =>
            Trace( This.Debug, "<Error> Unexpected symbol encountered <Error>");
            Trace( This.Debug, " at line " & Integer'Image(This.Current_Line_Count));
      end case;
   exception
      when TI.End_Error =>
         Trace( This.Debug, "<<<ERROR>>> End Error trap in Parse Design Library Unit.");
      when others =>
         Trace( This.Debug, "<<<ERROR>>> others trap in Parse Design Library Unit.");
   end Parse_Compilation_Unit;


   ---======================================================================---
   ---====== E X P O R T E D   P R O C E D U R E S / F U N C T I O N S =====---
   ---======================================================================---

   --|
   --|
   --|
   procedure Initialize( This  : in out Object'Class; 
                         debug : Debugging_Support.Handle := null ) is
      Data : Object_Data_Access := This.Data;
   begin
      if Data = null then
         This.Data := Initialize;
         Data := This.Data;
      end if;

      Data.Debug := Debug;
      Initialize( Data.Comments, debug );
      Initialize( Data.Result, debug );
      Initialize( Data.Libraries, debug );

   end Initialize;

   --|
   --|
   --|
   procedure Finalize( This : in out Object'Class ) is
      O : Object_Data_Access := This.Data;
   begin
      if O /= null then
         Free( O );        -- remove object data from memory
         This.Data := null;
      end if;
   end Finalize;

   --|
   --| analyze the given source module and build the entry list 
   --| for later processing
   --|
   procedure Parse( O : in out Object'Class; Ada_Source : in String ) is
      This   : Object_Data_Access := O.Data;
   begin
      if This = null then
         Raise_Exception(ASCL.Usage_Error'Identity,
                         "(ada_parser.parse) Object not initialized"); 
      end if;

      Open_New_Ada_Source_File ( O, Ada_Source );
      Parse_Context_Clause(O);
      Parse_Compilation_Unit(O);

   exception 
      when The_Error : others =>
         Raise_Exception(Exception_Identity(The_Error),
                         " ada_parser.parse at line" &
                         Integer'Image(This.Current_Line_Count));
   end Parse;

   --|
   --|
   --|
   function Result ( O : in Object'Class ) return List.Object is
      Data : Object_Data_Access := O.Data;
   begin
      return Data.Result;
   end Result;
   
end Parser;





