--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/restricted.adb,v $
--| Description     : Restricted objects
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--| Functional Description
--| ======================
--| The object instance maintains a list for each role. For 
--| execution of a command the list of the current role is checked.
--| If the command identifier is not contained in the access
--| list, an exception will be raised.
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: restricted.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--| Revision 1.1  2000/01/15 18:23:44  ACDE
--| No comments
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;
with Unchecked_Deallocation;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
use  ASCL;

with List;                         use List;
with Interface.List_Element;       use Interface.List_Element;
with Interface.Command;            use Interface.Command;
use  Interface;
with Simple_List_Element;

package body Restricted is

   Version : constant String := "$Id: restricted.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "restriced.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---
  	
   type Role_Array is array( Role_Identifier ) of List.Object;
   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Debug        : Debugging_Support.Handle := null;

	 Current_Role : Role_Identifier;
	 Access_List  : Role_Array;
      end record;

   ---=====================================================================---
   ---===     L O C A L  S U P P O R T   T Y P E S   AND   D A T A	===---
   ---=====================================================================---
   type Access_List_Record is record
         cmd   : Unbounded_String := Null_Unbounded_String;
	 where : Role_Identifier;
      end record;

   package Access_List_Pkg is 
         new Simple_List_Element( Item_Type=> Access_List_Record );
   use Access_List_Pkg;
   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data of the instance
   --|
   function Instance( this : Object'Class; info : String ) 
                                               return  Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;

   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data and call the Initialization
   --| procedure of the extension.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug; 
      for i in Role_Identifier loop
         Initialize( Data.Access_List(i), debug );
      end loop;

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object'Class ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;

      Free( data );

      this.data := null;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Return the name for a given role identifier
   --|
   function Role( this  : in Object'Class ) return Role_Identifier is
      data : Object_Data_Access := Instance( this, ".Role");
   begin
      return Data.Current_Role;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Role");
         raise;
   end Role;

   --|
   --| Set the role to a given name
   --|
   procedure Role( this : in out Object'Class; role : in Role_Identifier) is
      Data  : Object_Data_Access := Instance( this, ".Role");
      Debug : Debugging_Support.Handle renames Data.Debug;
   begin
      Enter( Debug, Module & "Role: " & Role_Identifier'Image(role));

      Data.Current_Role := role;

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave( Debug, "** exception **");
         Error( this, The_Error, ".Action" );
         raise;
   end Role;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Add an authorization to the object
   --|
   procedure Authorize( this  : in out Object'Class; 	
   			cmd   : in Command.External_ID;
                        where : in Role_Identifier)
   is
      Data  : Object_Data_Access := Instance( this, ".Authorize" );
      Debug : Debugging_Support.Handle renames Data.Debug;
      Item  : Access_List_Record;
   begin
      Enter( Debug, Module & ".Autorize");

      Item.Cmd   := To_Unbounded_String( cmd );
      Item.Where := where;

      Append( Data.Access_List(where), Construct( Item ).all );
      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave(Debug, "**exception**");
         Error( this, The_Error, ".Authorize" );
         raise;
   end Authorize;

   --|
   --|	Execute the given command assuming the currently set role.
   --|
   procedure Execute( this : in Object'Class; 
                      cmd  : in out Command.Object'Class ) is
      Data   : Object_Data_Access  := Instance( this, ".Execute" );

      Debug  : Debugging_Support.Handle renames Data.Debug;
      Role   : Role_Identifier          renames Data.Current_Role;
      Item   : List_Element.Handle      := null;
      Result : Access_List_Record;
   begin
      Enter( Debug, Module & ".Execute");

      Item := Get( Data.Access_List( Role ), First );
      while item /= null loop
	 Result := Value( Access_List_Pkg.Object(Item.all));
	 if To_String(Result.Cmd) = Id( cmd ) then
	    Handler( cmd );
	    return;
	 end if;

         Item := Get( Data.Access_List( Role ) );
      end loop;
      raise Not_Authorized;

   exception
      when Not_Authorized =>
      	 raise;

      when The_Error : Others =>
         Leave(Debug, "**exception**");
         Error( this, The_Error, ".Execute" );
         raise;
   end Execute;

end Restricted;

