--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/std-process.adb,v $
--| Description     : Base process for all CGI based ACDE processes
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This module implements the basic CGI process frame work.
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: std-process.adb,v $
--| Revision 1.2  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--| Revision 1.1  2000/01/20 20:59:07  ACDE
--| No comments
--|
--|

--- Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Strings.Unbounded;        use Ada.Strings.Unbounded;
with Ada.Text_IO;                  use Ada.Text_IO;
with Ada.Command_Line;             use Ada.Command_Line;
with Ada.Numerics.Discrete_Random;
with Unchecked_Deallocation;

--- Third party components
with CGI;        

--- ASCL Environment
with ASCL;			   use ASCL;
with ASCL.Debugging_Support;  	   use Debugging_Support;
with ASCL.App.Initfile;            use ASCL.App.Initfile;
with ASCL.DB;
use  ASCL.App;

--- ACDE Environment
with STD.CGI_Debug;
use  STD;
	        
with HTML;                         use HTML;	
with Environement; 		   use Environement;

with DBS.SQL;			   use DBS.SQL;
with DBS.T_Userlist;		   use DBS.T_Userlist;
use  DBS;

package body STD.Process is

   Version : constant String := "$Id: std-process.adb,v 1.2 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "std-process.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   type Component_Info;
   type Component_Info_Access is access Component_Info;

   type Component_Info is record
         Name        : Unbounded_String := Null_Unbounded_String;
	 Value       : Unbounded_String := Null_Unbounded_String;
	 Mode        : Component_Type   := Not_Defined;
	 Next        : Component_Info_Access := null;
      end record;
			 

   package Int_Random is new Ada.Numerics.Discrete_Random(Integer);
   use Int_Random;

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Debug          : Debugging_Support.Handle := null;
	 Component_list : Component_Info_Access    := null;
	 Config         : InitFile.Object;
	 dbase          : DB.Object;
	 Current_qID    : DB.Query_ID;
	 Name           : Unbounded_String         := Null_Unbounded_String;
	 Transaction    : Integer                  := 1;
	 Random_Number  : Generator;
      end record;

   --|
   --| Configuration variables from the initfile.
   --|
   DB_SECTION     : constant String := "db";
   DB_USER        : constant String := "serverUser";
   DB_SERVER      : constant String := "server";

   PROC_SECTION   : constant String := "process";    
   PROC_NAME      : constant String := "name";

   --|
   --| Fixed CGI Input Components. The following naming convention
   --| is used:
   --|
   --|    <x>_Name 
   --|
   --| where  X => P identifies a parameter and in case of X=>V
   --| a value of a variable which is expected from the 
   --| html from.
   --| 
   P_Action       : constant String := "Action";
   V_Submit	  : constant String := "Submit";
   V_Cancel       : constant String := "Cancel";
   P_Login_Name   : constant String := "Name";   
   P_Transaction  : constant String := "Transaction";
   P_Password     : constant String := "Password";

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---


   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String )
   is					       
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "<FONT COLOR=RED><B>*** Exception raised</B></FONT>" );

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;                                                       
	   
   --|
   --| Retrieve the object instance data
   --|
   function Instance_Data( this : Object'Class; info : String ) 
                                              return Object_Data_Access is
   begin
      if this.data = null then
         raise Not_Initialized;
      end if;
      Trace( this.Data.Debug, "Accessing instance data at " & info );

      return this.Data;
   exception
      when The_Error : Others =>
         Error( this, The_Error, Module & ".Instance_Data for " & info );
	 raise;
   end Instance_Data;

   --|
   --| Find an component in the component list
   --|
   function Find( this : in Object'Class; name : in String ) return Component_Info_Access is
      data    : Object_Data_Access    := this.data;
      Current : Component_Info_Access := null;
   begin
      Current := data.Component_List;
      while Current /= null loop
         exit when To_String(Current.Name) = name;

	 Current := Current.Next;
      end loop;

      return Current;
   end Find;

   --|
   --| Perform a login by checking the user id and the passrod in 
   --| the data base. 
   --|
   function Login( this   : in Object'Class;
                   name   : in String;
	           passwd : in String) return Boolean 
   is
      Data        : Object_Data_Access := Instance_Data( this, ".Login");
      Debug       : Debugging_Support.Handle renames Data.Debug;
      dBase       : DB.Object := Data_Base( this );
      qId         : DB.Query_ID;
      Result      : Boolean := False;
 
      function Trim( s : in String ) return String is
      begin
         return s(2..s'Length);
      end Trim;

    begin   
      Enter( Debug, Module & ".Login" );

      qID := DB.Query( dBase,
                 SQL_Select( 
		    from  => T_User_List,
	            where => D_User_Name & "='" & name & "'" & " and " &
			     D_Password  & "='" & passwd & "'" 
	         )
	     );			   
      if DB.Nbr_Of_Rows(dBase,qID) < 1  then
         Result := False;
      else
         Result := True;	     
      end if;
      DB.Drop_Query( dBase, qID ); 

      if Result = True then
         begin
            Trace( Debug, "Updating transaction");
            qID := DB.Query( dBase,
	        SQL_Update( 
		    update => T_User_List,
		    colum  => D_Transaction & "=" &
	 	              Trim(Integer'Image(Random( Data.Random_Number)))
	        )); 
	 
            DB.Drop_Query( dBase, qID );
	 exception
	    when DB.No_Query_Result =>
	       Trace(debug, "No query Result");
         end;
      end if;

      Leave( Debug, " result = " & Boolean'Image(Result));
      return Result;

   exception
      when DB.No_Query_Result =>
      	 return False;
      when The_Error : Others =>
         Leave( Debug, "Exception" );
         Error( this, The_Error, ".Login");
	 raise;
   end Login;

   --|
   --| Perform a login by checking the user id and the passrod in 
   --| the data base. 
   --|
   function Transaction_ID( this : Object'Class; name : String ) return Integer is
      Data     : Object_Data_Access := Instance_Data( this, ".Transaction_ID");
      Debug    : Debugging_Support.Handle renames Data.Debug;
      dBase    : DB.Object := Data_Base( this );
      qId      : DB.Query_ID;
      Result   : Integer;
    begin   
      Enter( Debug, Module & ".Transaction_ID" );

      qID := DB.Query( dBase,
      		SQL_Select( 
		   colum => D_Transaction,
		   from  => T_User_List,
		   where => D_User_Name & "='" & name & "'" 
		)
	     );				   
      if DB.Nbr_Of_Rows(dBase,qID) = 0 then
         Result := 0;
      else
         Result := DB.Integer_Field(dBase, qID, "transaction" );
      end if;
      DB.Drop_Query( dBase, qID ); 

      Leave( Debug, " result = " & Integer'Image(Result));
      return Result;

   exception
      when The_Error : Others =>
         DB.Drop_Query( dBase, qID );
         Leave( Debug, "Exception" );
         Error( this, The_Error, ".Transaction_ID");
	 raise;
   end Transaction_ID;

   --|
   --| Find out the UNIX style base name. Proably it is in some library 
   --| available but ...
   --|
   function Base_Name( s : String ) return String is
      idx : Integer := s'Length;
   begin
      while idx > 0 loop
         if s(idx) = '/' then
	    return s(idx+1..s'Length);
	 end if;
	 idx := idx - 1;
      end loop;

      return s;
   end Base_Name;
		    
   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Initialize the Object data by allocating the object memory
   --| and calling the initialize procedures of the included
   --| components.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data : Object_Data_Access renames this.Data;
   begin
      Enter( debug, Module & ".Initialize");
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;
      this.Data := Data;

      --- Perform local initialization
      Data.Debug := debug;
      Reset( Data.Random_Number );

      --- open the configuration file
      Initialize( Data.Config );
      Get( Data.Config, ACDE_Config_PATH & 
                        "/" & 
			Base_Name( Command_Name ) &
			".ini" );

      --- initialize other packages
      DB.Initialize( data.dBase, Debug );
      HTML.Initialize( Debug );
      SQL.Initialize( Debug );

      Leave( debug, "" );
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extension.
   --|
   procedure Finalize( this : in out Object'Class ) is
      data   : Object_Data_Access renames this.data;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      if data = null then
         return;
      end if;
      Enter( data.debug, Module & ".Finalize" );

      Finalize( Data.Config );
      DB.Finalize( Data.dBase );

      Free( data );
      this.data := null;

      Leave( data.Debug, "" );
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Retrieve a comonent value
   --|
   function Component( this : in Object'Class; 
                       Name : in String ) return Unbounded_String 
   is
      data    : Object_Data_Access := Instance_Data( this, ".Component");
      Current : Component_Info_Access := null;
   begin
      Enter( data.Debug, Module & ".Component: " & Name );

      Current := Find( this, Name );
      if Current /= null then   
         Leave( data.debug, " result =" & To_String( Current.Value ) );
         return Current.Value;
      else 
         raise Usage_Error;
      end if;
     
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Component (get)");
	 raise;
   end Component;

   --|
   --| Add a component. If the component is allready defined create
   --| an exception. 
   --|
   procedure Component( this : in out Object'Class; 
                        name : in String; 
			mode : in Component_Type := Optional ) 
   is
      Data    : Object_Data_Access := Instance_Data( this, ".Component");
      Current : Component_Info_Access := null;
   begin
      Enter( data.Debug, Module & ".Component" );

      if Find( this, name ) /= null then
         raise Usage_Error;
      end if;

      Current := new Component_Info;
      Current.Name := To_Unbounded_String( name );
      Current.Mode := mode;

      if Data.Component_List /= null then
         Current.Next := Data.Component_List;
      end if;
      Data.Component_List := Current;
           
      Leave( data.debug, "" );

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Component (set)" );
   end Component;

   --|
   --| Retrieve the current debug pointer
   --|
   function Debug( this : in Object'Class) return Debugging_Support.Handle is 
      data    : Object_Data_Access := Instance_Data( this, ".Debug");
   begin
      Enter( data.Debug, Module & ".Debug" );

      Leave( data.Debug, "" );
      return Data.Debug;
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Debug (get)");
	 raise;
   end Debug;

   --|
   --| Return the current data base object
   --|
   function Data_Base( this : in Object'Class ) return DB.Object is
      Data : Object_Data_Access := this.Data;
   begin
      return Data.dBase;
   end Data_Base;
  			 
   --|
   --| Return the current Query. This procedure is inteded to be 
   --| used in combination with the query prcoedure.
   --|
   function Current_Query( this : in Object'Class ) return DB.Query_ID is
      Data : Object_Data_Access := this.Data;
   begin
      return Data.Current_qID;
   end Current_Query;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Purpose:
   --|   perform a query an walk trough the query by calling a 
   --|   user provided procedure.
   --|
   procedure Query( this         : in out Object'Class; 
                   query_string  : in String;
		   Handle_Result : in Query_Handler  ) 
   is
      dBase          : DB.Object := Data_Base( this );
      qId            : DB.Query_ID;
      Current        : Integer := 0;
      Data           : Object_Data_Access := Instance_Data(this, ".Query");
      Debug          : Debugging_Support.Handle renames Data.Debug;
      Continue_Query : Boolean := True;
   begin   
      Enter( Debug, Module & ".Query" );

      Trace( debug, "Performing query: " & query_string );
      qID := DB.Query( dBase, query_string );
      if DB.Nbr_Of_Rows( dBase, qID ) = 0 then
         DB.Drop_Query( dBase, qID );
	 return;
      else 
         Data.Current_qID := qID;
      end if;


      while Continue_Query loop
         begin
	    Current := Current + 1;

	    Trace( Debug, "Calling Result handler for:" & Integer'Image(Current));
	    Continue_Query := Handle_Result( this );
            DB.Next( dBase, qID );

	 exception
	    when DB.Field_Parse_Error =>
	       Trace( Debug, "Field parse error at:" & Integer'Image(Current));
	       DB.Next( dBase, qID );
	    when DB.No_More_Rows =>    
	       Trace( Debug, "No more Tuples at:" & Integer'Image(Current));
	       DB.Drop_Query( dBase, qID );
	       exit;
	    when Others =>
	       raise;
	 end;
      end loop;
      Leave( Debug, "");

   exception
      when The_Error : Others =>
         DB.Drop_Query( dBase, qID );
         Leave( Debug, "Exception at tuple" & Integer'Image(Current) );
         Error( this, The_Error, ".Query (query)");
   end Query;

   --|
   --| Purpose:	        
   --|    This procedure builds a standard html page for the process. The 
   --|    custom part of the page is passed as an pointer to a procedure.
   --|
   procedure Page( this            : in out Object'Class;
                   Customized_Page : Customized_Page_Handler;
		   title           : in String  ) 
   is
      Data  : Object_Data_Access := Instance_Data( this, ".Page" );
      Debug : Debugging_Support.Handle renames Data.Debug;
      Name  : Unbounded_String renames Data.Name;
   begin    
      Form( Action => URL_Base & "/" & Base_Name( Command_Name ) ); 
      HTML.Table(" width=99% COLS=1 bgcolor=" & HTML_String("#cccc99") );
         ----------------------------------------------------------------------
	 ---                 CONTEXT PART OF THE FORM	  	       ---
 	 ----------------------------------------------------------------------      
         HTML.TR( "VALLIGN=MIDDLE ALLIGN=CENTER" );
            HTML.TD;
               HTML.Heading( 1,title);  
	    HTML.TD_End;
	 HTML.TR_End;

	 HTML.TR;
	    HTML.TD;
	       if Customized_Page /= null then
	          Customized_Page( this );
	       else
	          HTML.Print("<B>*** Not yet implemented ***</B>");
	       end if;
	    HTML.TD_End;
	 HTML.TR_End;

	 HTML.TR( " VALIGN=MIDDLE ALIGN=CENTER " );
	    HTML.TD;
	       Break;
	       HTML.Submit( Value => V_Submit , name => P_Action);
	       HTML.Print("&nbsp;&nbsp;");
	       HTML.Submit( Value => V_Cancel, name => P_Action );
               HTML.Print("&nbsp;&nbsp;");
	       HTML.Reset("RESET");
	       Break;
	    HTML.TD_End;
	 HTML.TR_End;
	   
      HTML.Table_End;

      HTML.Line;
      HTML.Print("<FONT COLOR=" & HTML_String("#cccccc") & ">"	);
      HTML.Print( "Logged in user : " );
      HTML.Text( Name => P_Login_Name, Value => To_String( Name ) );
      HTML.Print("&nbsp;&nbsp;");
      HTML.Print("Transaction : ");
      HTML.Text( Name  => P_Transaction,
	               Value => Positive'Image(Data.Transaction));
      HTML.Print("</FONT>");
      HTML.Line;

      Form_End;
   end Page;

   --|			  
   --| Purpose:
   --|    This is the main working procedure of a ACDE process. It sets
   --|    up the operational environment and calls the Prepare and 
   --|    Execute procedure of the process it self.      
   --|
   procedure CGI_Process( this : in out Object'Class ) is
      Data    : Object_Data_Access := Instance_Data( this, ".CGI_Process");
      Current : Component_Info_Access := null;
      Debug   : Debugging_Support.Handle renames Data.Debug;
      Logging : File_Type;
   begin
      Enter( Data.Debug, Module & ".CGI_Process");

      CGI.Put_CGI_Header;

      --- ======================================================= ---
      --- ===              CONNECT TO DATA BASE               === ---
      --- ======================================================= ---
      Section( Data.Config, DB_SECTION );
      Trace( Debug, 
             "Connecting to " & Value( Data.Config, DB_SERVER ) &
	     " as " & Value( Data.Config, DB_USER));
      DB.User( Data.dBase, Value( Data.Config, DB_USER ));
      DB.Connect( Data.dBase, Value( Data.Config, DB_SERVER) );
      DB.Select_DB( Data.dBase, ACDE_DB_Name );
  
      --- ====================================================== ---
      --- ===             PROCESS THE INPUT DATA             === ---
      --- ====================================================== ---
      if CGI.Input_Received then
         Trace( Debug, "CGI Input recevied" );

    	 -- verfiy if the user is valid. 
	 if CGI.Key_Exists( P_Transaction )  then
	    Trace( Debug, "Login via transaction identifier");
	    if Integer'Value(To_String(CGI.Value(P_Transaction))) /= 
	       Transaction_ID(this, CGI.Value(P_Login_Name)) then
	       raise Login_Failure;
	    else
	       Data.Transaction := 
	         Integer'Value(To_String(CGI.Value(P_Transaction)));
	       Data.Name := CGI.Value( P_Login_Name );
	    end if;
	 elsif Login( this, CGI.Value(P_Login_Name), CGI.Value(P_Password)) then
	    Data.Name := CGI.Value( P_Login_Name );
	    Data.Transaction := Transaction_ID( this, CGI.Value( P_Login_Name ));
	    Trace(Debug, "Login successfull");
	 else
	    raise Login_Failure;
	 end if;

	 -- Prepare data
	 Prepare( this );

	 -- check for all mandatory parameters
	 Current := Data.Component_List;
	 while Current /= null loop
	    Current.Value := CGI.Value( To_String(Current.Name) );
	    if To_String( Current.Value ) = "" and Current.Mode = Mandatory  then
	       Trace( Debug, 
	              "<B>*** Missing CGI Input: <FONT COLOR=RED>" & 
		      To_String(Current.Name) & "</FONT></B>");
	       raise Usage_Error;
	    end if;
					       	
	    Current := Current.Next;
	 end loop;
	 Trace( data.debug, "CGI parameters verified");

	 Section( Data.Config, PROC_SECTION );
         CGI.Put_HTML_Head( "ACDE - " & Value( Data.Config, PROC_NAME ));

	 --
	 -- check if an action is to be performed or the page has to
	 -- be build.
	 --
 	 if CGI.Key_Exists( P_Action ) then declare
	       action : Unbounded_String := CGI.Value( P_Action );
	    begin
	       if To_String( action ) = V_Submit then
	          Perform( this ); 
		  Display( this );
	       elsif To_String( action ) = V_Cancel then
	          Cancel( this );
		  Display( this );
	       end if;
	    end;
	 else
	    Display( this );
	 end if;

         CGI.Put_HTML_Tail;
      else
          Trace( Debug, "No CGI parameter recevied");
          raise Usage_Error;
      end if;

      Leave( Debug, "");

      Create( File => Logging, 
              Name => "/tmp/logging", 
              Mode => Out_File );
      CGI_Debug.Dump( CGI_Debug.Object(Debug.all), Logging );  
      Close( Logging );

   exception
      ---***************************************---
      ---***       HANDLE EXCEPTIONS         ***---
      ---***************************************--- 
      when Login_Failure =>
         CGI.Put_HTML_Head("Login Failure");
	 Put_Line("Login faulure");  
	 CGI.Put_HTML_Tail;            
      when The_Error : Others =>
         Leave( Debug, "<FONT COLOR=RED><B>Exception</B></FONT>");
         CGI.Put_HTML_Head("Processing Error");      
         Put_Line(
            "<HR><B>*** Exception <FONT COLOR=RED>" 
	    & Exception_Name( The_Error ) &
            "</FONT> in " & Module & ".CGI_Process </B><BR><BR>");

	 Debugging_Support.Error( Debug, "*** Internal failure ***");
	 Break;
	 CGI_Debug.Dump( CGI_Debug.Object(Debug.all), Standard_Output );  

         CGI.Put_HTML_Tail;         
   end CGI_Process;

end STD.Process;

