--|  
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/std-process.ads,v $
--| Description     : ACDE cgi base process
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Version         : $Revision: 1.2 $
--| Status          : $State: Released $
--|
--| Functional Description
--| ======================
--| This is the base class for all ACDE cgi based processes.  The extension of the 
--| class will see an simplified interface towards the CGI and the DB Interface.
--|
--| For further discussion refere to the acde design document regarding 
--| acde base processes.
--|			     
--| Component Data
--| ==============
--| Component - A cgi parameter string named component. Name and value are
--|             made available by the component api.
--| Debug     - The debugging handle of the base process. This has to be used
--|             by the implementation in order to use debugging support.
--| Data_Base - The data base object
--| Current_Query
--|           - The current query as allocated by the query method.
--|
--| Operations on the component data
--| ================================
--| CGI_Process
--|
--| Error Handling
--| ==============
--|
--| Extension
--| =========
--| Prepare
--| Display
--| Perform
--| Cancel
--|
--| Restrictions
--| ============
--| Tasking: no
--| Y2K    : dc
--|			 
--| History
--| =======
--| $Log: std-process.ads,v $
--| Revision 1.2  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--| Revision 1.1  2000/01/20 20:59:18  ACDE
--| No comments
--|
--| Revision 1.9  2000/01/16 16:03:56  ACDE
--| Release Version V8_1test
--|
--| Revision 1.8  2000/01/16 16:02:06  ACDE
--| Release Version V8test
--|
--| Revision 1.7  2000/01/16 15:59:51  ACDE
--| Release Version V7test
--|
--| Revision 1.6  1999/12/18 13:29:00  ACDE
--| Release Version V6test
--|
--| Revision 1.5  1999/12/18 13:22:58  ACDE
--| Release Version V5test
--|
--| Revision 1.4  1999/12/18 13:20:45  ACDE
--| Release Version V4test
--|
--| Revision 1.3  1999/12/18 13:17:53  ACDE
--| Release Version V3test
--|
--| Revision 1.2  1999/12/18 11:39:29  ACDE
--| Release Version V2test
--|
--| Revision 1.1  1999/12/18 11:38:03  ACDE
--| Release Version V1test
--|
--| Revision 1.1  1999/12/05 19:28:07  ACDE
--| documentation added
--|
--|
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;

with ASCL.Debugging_Support;
with ASCL.DB;
use  ASCL;

package STD.Process is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is abstract tagged limited private;
   type Handle is access Object'Class;

   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null );

   procedure Finalize( this : in out Object'Class );

   type Query_Handler is access 
       function( this : in Object'Class ) return Boolean;

   Login_Failure : exception;
   ---=====================================================================---
   ---===                  A T T R I B U T E S                          ===---
   ---=====================================================================---

   type Component_Type is ( Mandatory, Optional, Not_Defined );

   ---------------------------------------------------------------------------
   --| Description    : 
   --|   Define a component with a certain mode. 
   --|       Mandatory 
   --|       Optional
   --| Preconditions  :
   --|   1. Component name is not yet defined
   --|   2. Object is initialized
   --| Postconditions : -
   --| Exceptions     : 
   --|       Usage_Error
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Component( this : in out Object'Class; 
                        name : in String; 
			mode : in Component_Type := Optional);


   ---------------------------------------------------------------------------
   --| Description    :
   --|    Get the value of the component.
   --| Preconditions  :
   --|   1. object is initialized
   --|   2. definition for component exists.
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Component( this : in Object'Class;
                       name : in String ) return Unbounded_String;

   ---------------------------------------------------------------------------
   --| Description    :
   --|   Return the debug handle
   --| Preconditions  :
   --|   1. Object has been initialized
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   function Debug( this : in Object'Class ) return Debugging_Support.Handle;

   function Data_Base( this : in Object'Class ) return DB.Object;
   function Current_Query( this : in Object'Class ) return DB.Query_ID;
   
               
   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    :
   --| Preconditions  :
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure CGI_Process( this : in out Object'Class );  


   procedure Query( this          : in out Object'Class; 
                    query_string  : in String;
                    Handle_Result : in Query_Handler );


   type Customized_Page_Handler is 
         access procedure( this : in out Object'Class );

   procedure Page( this            : in out Object'Class;
                   Customized_Page : Customized_Page_Handler;
		   title           : in String  );

   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---
   procedure Prepare( this : in out Object ) is abstract;
   procedure Display( this : in out Object ) is abstract;
   procedure Perform( this : in out Object ) is abstract;

   procedure Cancel( this : in out Object ) is abstract;

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is abstract tagged limited record
         data : Object_Data_Access := null;
      end record; 

end STD.Process;


