--|
--| Filename        : $Source: /home/ACDE/ACDE/Source/RCS/text_block.adb,v $
--| Description     : Handle a Text block.
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 2000/01/21 17:18:23 $
--| Status          : $State: Released $
--| Version         : $Revision: 1.1 $
--|
--| Functional Description
--| ======================
--| This package simply stores a block of text lines in a list.
--|
--| Restrictions
--| ============
--|
--| References
--| ==========
--|
--|
--| History
--| =======
--| $Log: text_block.adb,v $
--| Revision 1.1  2000/01/21 17:18:23  ACDE
--| Release Version test_1_0
--|
--|

--* Ada
with Ada.Exceptions;               use Ada.Exceptions;
with Ada.Strings.Unbounded;	   use Ada.Strings.Unbounded;

with Unchecked_Deallocation;

with ASCL.Debugging_Support;  	   use ASCL.Debugging_Support;
use  ASCL;

with Interface.List_Element;	   use Interface;
with List;			   use List;

with Simple_List_Element;

package body Text_Block is

   Version : constant String := "$Id: text_block.adb,v 1.1 2000/01/21 17:18:23 ACDE Released ACDE $";
   Module  : constant String := "text_block.adb";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         Debug        : Debugging_Support.Handle := null;
	 Nbr_Of_Lines : Integer := 0;
	 Text         : List.Object;
      end record;

   ---=====================================================================---
   ---===               L O C A L  D A T A  / T Y P E S 		===---
   ---=====================================================================---   
   package Text_Line is new Simple_List_Element( Item_Type => Unbounded_String );
   use Text_Line;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---

   --|
   --| This procedure is called upon every exception raised in this
   --| component. It may be used to trace back the cause of an
   --| exception, or to print out relevant information about the
   --| instance where the problem did occure.
   --|
   procedure Error( this     : in Object'Class;
                    theError : in Exception_Occurrence;
                    info     : in String ) 
   is
      data : Object_Data_Access := this.data;
   begin
      Trace( data.Debug, "*** Exception : " & Exception_Name( theError ) );
      Trace( data.Debug, "              : " & Exception_Message( theError) );
      Leave( data.Debug, "*** Exception ***");

      Raise_Exception( Exception_Identity( theError ),  Module & info );
   end Error;

   --|
   --| Get pointer to object data
   --|
   function Instance( this : Object'Class; info : String ) 
                                               return  Object_Data_Access is
   begin
      if this.Data = null then
         raise Not_Initialized;
      end if;

      return this.Data;
   exception
      when The_Error: Others => 
         Error( this, The_Error, ".Instance");
	 raise;
   end Instance;

   --|
   --|
   --|

   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---

   --|
   --| Intialize the Object data and call the Initalization
   --| procedure of the extention.
   --|
   procedure Initialize( this  : in out Object'Class;
                         debug : in Debugging_Support.Handle := null ) 
   is
      Data  : Object_Data_Access renames this.data;
   begin
      Enter( debug, Module & ".Initialize" );
      if Data /= null then
         raise Usage_Error;
      end if;

      Data := new Object_Data;
      if Data = null then
         raise Out_Of_Memory;
      end if;

      Data.Debug := debug;
      
      Initialize( Data.Text, Debug ); 

      Leave( debug, "");
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Initialize");
   end Initialize;

   --|
   --| Finalize the instance by releasing the object data
   --| finalizing the extention.
   --|
   procedure Finalize( this : in out Object'Class ) is
      Data   : Object_Data_Access := Instance( This, ".Finalize");
      Debug  : Debugging_Support.Handle := Data.Debug;

      procedure Free is
            new Unchecked_Deallocation( Object_Data, Object_Data_Access);
   begin
      Enter(Debug, Module & ".Finalize" );
       
      Finalize( Data.Text );

      Free( data );
      this.data := null;

      Leave( Debug, "");

   exception
      when The_Error : Others =>
         Leave( Debug, "** exception **");
         Error( this, The_Error, ".Finalize");
   end Finalize;

   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Yield the number of lines inserted into the text block.
   --|
   function Length( this : in Object'Class ) return Integer is
      Data : Object_Data_Access := Instance( this, ".Get");
   begin
      return Data.Nbr_Of_Lines;

   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Attribute");
         raise;

   end Length;

   --|
   --| Return the stored text as a string array.
   --|
   function Text ( this : in Object'Class ) return Text_Array_Access is
      Data   : Object_Data_Access := Instance( this, ".Text");
      Debug  : Debugging_Support.Handle renames Data.Debug;

      Result : Text_Array_Access := null;
        
      procedure Add( Pos  : in out Integer; 
                     item : in out List_Element.Object'Class ) is
      begin
         Result(Pos) := Value( Text_Line.Object(item) );
	 Pos := Pos + 1;  
      end Add;

      procedure Copy is 
               new List.Perform( Iterator_Type => Integer, Process => Add );

      Line_Number : Integer := 1;
   begin
      Enter( Debug, Module & ".Text" );

      Result := new Text_Array( 1..Length( This ) );
      Copy( Data.Text, Line_Number );

      Leave( Debug, "" );
      return Result;	 

   exception 
      when The_Error : Others =>
         Leave( Debug, "** exception **");
         Error( this, The_Error, ".Text");
         raise;

   end Text;
   --|
   --| Set some attribute
   --|
   procedure Set( this : in out Object'Class; value : in Integer ) is
      data : Object_Data_Access := Instance( this, ".Get");
   begin
      null;
   exception
      when The_Error : Others =>
         Error( this, The_Error, ".Action" );
         raise;
   end Set;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Add a text line to the text block. The new line will be placed
   --| at the end of the text block.
   --|
   procedure Add( this : in out Object'Class; text : in String ) is
      Data  : Object_Data_Access := Instance( this, ".Add" );
      Debug : Debugging_Support.Handle renames Data.Debug;

      Item  : List_Element.Handle := null;
      Line  : Unbounded_String;
   begin
      Enter( Debug, Module & ".Add");

      Line := To_Unbounded_String( text );

      Item := Construct( Line );
      Append( Data.Text, Item.all );

      Data.Nbr_Of_Lines := Data.Nbr_Of_Lines + 1;

      Leave( Debug, "");
   exception
      when The_Error : Others =>
         Leave(Debug, "**exception**");
         Error( this, The_Error, ".Add" );
         raise;
   end Add;

end Text_Block;

